/* $Id: e2_upgrade.c,v 1.5 2004/08/07 21:57:53 cairo Exp $
 * 
 * Copyright (C) 2003-2004 tooar <tooar@gmx.net>
 * 
 * This file is part of emelFM2.
 * emelFM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * emelFM2 is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with emelFM2; see the file COPYING. If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include "emelfm2.h"
#include "e2_upgrade.h"
#include "e2_option.h"
#include "e2_output.h"
#include "debug.h"
#include "e2_dialog.h"
#include "build.h"
#include "e2_button.h"
#include "e2_cl_option.h"

/**
 * @file src/config/e2_upgrade.c 
 * @brief configuration upgrade functions
 *
 * This file contains functions that help upgrading the configuration file
 * between different e2 versions.
 **/

void e2_upgrade_001 ()
{
	printd (DEBUG, "upgrade from v0.0.1");
	add_filetype("<directory>", "cd@cd %%d/%%f", "directories");
	add_filetype("<executable>", "run@./%f", "executables");
	status_message2 ("\nconfig was upgraded from version 0.0.1\n", TRUE, "bold", NULL);
	status_message2 ("   (added <directory> and <executable> filetypes)\n", TRUE, NULL);
	write_filetypes_file ();
}

void e2_upgrade_002 ()
{
	printd (DEBUG, "upgrade from v0.0.2");
	gchar *sed = g_find_program_in_path ("sed");
	if (sed == NULL)
	{
		printd (ERROR, "can't find 'sed' and thus i can't upgrade the config files");
		exit (1);
	}
	gchar *cfg_file = g_build_filename (e2_cl_options.config_dir, DEFAULT_CONFIG_FILE, NULL);
	gchar *command = g_strconcat ("cp ", cfg_file, " ", cfg_file, ".002; ",
		sed, " -r 's/(.*\\|.*)\\|(.*\\|.*\\|.*)/\\1||\\2/' ",
		cfg_file, " >", cfg_file, ".e2-upgrade; ",
		"mv ", cfg_file, ".e2-upgrade ", cfg_file, NULL);
	system (command);
	g_free (command);
	g_free (sed);
	g_free (cfg_file);
	status_message2 ("\nconfig was upgraded from version 0.0.2\n", TRUE, "bold", NULL);
	status_message2 ("   (added icon column to context-menu tree)\n", TRUE, NULL);
	e2_upgrade_003 ();
}

void e2_upgrade_003 ()
{
	printd (DEBUG, "upgrade from v0.0.3");
	gchar *sed = g_find_program_in_path ("sed");
	if (sed == NULL)
	{
		printd (ERROR, "can't find 'sed' and thus i can't upgrade the config files");
		exit (1);
	}
	gchar *cfg_file = g_build_filename (e2_cl_options.config_dir,
		DEFAULT_CONFIG_FILE, NULL);
	gchar *command = g_strconcat ("cp ", cfg_file, " ", cfg_file, ".003; ",
		sed, " -r 's/^(.*)\\|(.*)\\|(.*)\\|(.*)\\|(.*)$/\\1|\\3|\\4|\\2|\\5/' ",
		cfg_file, " >", cfg_file, ".e2-upgrade; ",
		sed, " -r 's/^(.*)\\|(.*)\\|(.*)\\|(.*)\\|(.*)\\|(.*)$/\\1|\\5|\\3|\\4|\\2|\\6/' ",
		cfg_file, ".e2-upgrade >", cfg_file, NULL);
	system (command);
	g_free (command);
	g_free (sed);
	g_free (cfg_file);
	status_message2 (_("\nconfig was upgraded from version 0.0.3\n"), TRUE,
		"bold", NULL);
	status_message2 (_("   (moved action column for tree options to 2nd last "
		"place)\n"), TRUE, NULL);
	GtkWidget *dialog = e2_dialog_create (GTK_STOCK_DIALOG_INFO,
		_("The config was upgraded to the current version. emelFM2 has to be "
		"restarted now.\nPress 'ok' to exit the program and start it again please."),
		_("update information"), NULL, NULL);
	e2_dialog_show (dialog, app.main_window, E2_BUTTON_OK,
		E2_DIALOG_RUN | E2_DIALOG_FREE);
	write_config_file ();
	exit (0);
}

void e2_upgrade_006 ()
{
	printd (DEBUG, "upgrade from v0.0.6");
	gchar *awk = g_find_program_in_path ("awk");
	if (awk == NULL)
	{
		printd (ERROR, "can't find 'awk' and thus i can't upgrade the config files");
		exit (1);
	}
	gchar *cfg_file = g_build_filename (e2_cl_options.config_dir,
		DEFAULT_CONFIG_FILE, NULL);
	gchar *command = g_strconcat ("cp ", cfg_file, " ", cfg_file, ".006; ",
		awk, " 'BEGIN { outfile=\"", cfg_file , ".upgrade\"; found=false; } "
		"{ if ($0==\"<command-aliases\") { print \"<aliases-removed\" >outfile; "
		"outfile=\"/dev/null\"; found=true }; if ((found==true) && ($0==\">\")) "
		"outfile=\"", cfg_file, ".upgrade\"; print > outfile; }' ", cfg_file,
		"; mv ", cfg_file, ".upgrade ", cfg_file, NULL);
	printd (DEBUG, "upgrade-command: %s", command);
	system (command);
	g_free (command);
	g_free (cfg_file);
	g_free (awk);
	status_message2 (_("\nThe configuration was upgraded from version 0.0.6\n"),
		TRUE, "bold", NULL);
	status_message2 (_("\t(The aliases have been replaced by their "
		"defaults again)\n"), TRUE, NULL);
/*	GtkWidget *dialog = e2_dialog_create (GTK_STOCK_DIALOG_INFO,
		_("The configuration has been upgraded to the current version. "
		"emelFM2 has to be restarted now.\nPress 'ok' to exit the program"
		"and start it again, please."), "update information", NULL, NULL);
	e2_dialog_show (dialog, app.main_window, E2_BUTTON_OK,
		E2_DIALOG_RUN | E2_DIALOG_FREE); */
//	write_config_file ();
	e2_upgrade_008 ();
}

void e2_upgrade_008 ()
{
	printd (DEBUG, "upgrade from v0.0.8");
	GtkWidget *dialog = e2_dialog_create (GTK_STOCK_DIALOG_INFO,
		_("This is the first time you run version 0.0.9 of emelFM2. The names of "
		"several actions have changed. Thus it is recommended you revert your "
		"toolbar and commandbar to their new defaults. Do you want to revert them?\n\n"
		"<small>(Note: After pressing 'yes' emelFM2 will quit and you will have to "
		"restart it)</small>"),
		"update information", NULL, NULL);
	gint response = e2_dialog_show (dialog, app.main_window,
		E2_BUTTON_NO | E2_BUTTON_YES, E2_DIALOG_RUN | E2_DIALOG_FREE);
	if (response == GTK_RESPONSE_YES)
	{
		e2_option_tree_revert ("toolbar");
		e2_option_tree_revert ("commandbar");
		status_message2 (_("\nThe toolbar and commandbar have been reverted to "
			"their default configuration.\n"), TRUE, "bold", NULL);
		printd (DEBUG, "toolbar and commandbar have been reverted to "
			"their default configuration");
		quit_cb (NULL);
	}
	exit (0);
}

void e2_upgrade_check (gboolean first_run)
{
	RUN_ONCE_CHECK ();

	if (!first_run)
	{
		const gchar *home = g_get_home_dir ();
		gchar *fn = g_build_filename (home, ".emelfm2", "config", NULL);
		if (((g_file_test (fn, G_FILE_TEST_EXISTS)) != TRUE) &&
			(!g_str_equal (cfg.version, VERSION)))
			e2_upgrade_001 ();
		g_free (fn);
		if (g_str_equal (cfg.version, "0.0.2"))
			e2_upgrade_002 ();
		if (g_str_equal (cfg.version, "0.0.3"))
			e2_upgrade_003 ();
		if (g_str_equal (cfg.version, "0.0.6"))
			e2_upgrade_006 ();
		if (g_str_equal (cfg.version, "0.0.8"))
			e2_upgrade_008 ();
	}
}
