/* $Id: e2_select_image_dialog.c,v 1.9 2004/08/14 12:01:08 cairo Exp $
 *
 * Copyright (C) 2003-2004 tooar <tooar@gmx.net>
 *
 * This file is part of emelFM2.
 * emelFM2 is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * emelFM2 is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with emelFM2; see the file COPYING. If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/**
 * @file src/dialogs/e2_select_image_dialog.c 
 * @brief Select image dialog
 *
 * This files contains all functions needed to create a select image dialog.
 **/

#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include "emelfm2.h"
#include "e2_select_image_dialog.h"
#include "debug.h"
#include "build.h"
#include "e2_dialog.h"
#include "e2_button.h"

  /*********************/
 /***** callbacks *****/
/*********************/

static void response_cb (GtkWidget *dialog, gint response, E2_SID_Runtime *rt)
{
	if (rt->page == 0)
	{
		if (response == 110)
		{
			g_object_set_data_full (G_OBJECT (dialog), "image", g_strdup (""),
				(GDestroyNotify) g_free);
			gtk_file_chooser_set_filename (GTK_FILE_CHOOSER (rt->file_chooser), "");
		} else
		{
			gchar *filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (rt->file_chooser));
			if (filename == NULL)
				filename = g_strdup ("");
			g_object_set_data_full (G_OBJECT (dialog), "image", filename,
				(GDestroyNotify) g_free);
		}
	} else
	{
		GtkTreeIter iter;
		GtkTreeModel *model;
		GtkTreeSelection *selection = gtk_tree_view_get_selection (rt->treeview);
		gboolean sel = gtk_tree_selection_get_selected (selection, &model, &iter);

		if (response == 110)
		{
			g_object_set_data_full (G_OBJECT (dialog), "image", g_strdup (""),
				(GDestroyNotify) g_free);
			if (sel)
				gtk_tree_selection_unselect_iter (selection, &iter);
		} else
		{
			if (sel)
			{
				gchar *icon;
				gtk_tree_model_get (model, &iter, 0, &icon, -1);
				g_object_set_data_full (G_OBJECT (dialog), "image", g_strdup (icon),
					(GDestroyNotify) g_free);
				g_free (icon);
			} else
				g_object_set_data_full (G_OBJECT (dialog), "image", g_strdup (""),
					(GDestroyNotify) g_free);
		}
	}
}

static void activated_cb (GtkTreeView *treeview, GtkTreePath *path,
	GtkTreeViewColumn *tree_column, GtkWidget *dialog)
{
	gtk_dialog_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK);
}

static void file_activated_cb (GtkTreeView *treeview, GtkWidget *dialog)
{
	printd (DEBUG, "file_activated_cb");
	gtk_dialog_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK);
}

static void switch_page_cb (GtkNotebook *notebook, GtkNotebookPage *npage,
	guint page, E2_SID_Runtime *rt)
{
	rt->page = page;
}

static void destroy_cb (GtkWidget *dialog, E2_SID_Runtime *rt)
{
	g_object_set_data (G_OBJECT (rt->parent), rt->name, NULL);
	g_free (rt->name);
	g_free (rt->icon);
	g_free (rt);
}

static void destroy_cb2 (E2_SID_Runtime *rt)
{
	gtk_widget_destroy (rt->dialog);
}

/*
//filter backup files from a GtkFileChooser listing
static gboolean image_files_filter (const GtkFileFilterInfo *filter_info,
	gpointer data)
{
	gsize len = filter_info->display_name ? strlen (filter_info->display_name) : 0;
	if (len > 0)
	{
		if (g_str_has_suffix (filter_info->display_name, "~"))
		return FALSE;
	} else
		return TRUE;
}
*/

static void update_preview_cb (GtkFileChooser *file_chooser, GtkWidget *preview)
{
	gchar *filename = gtk_file_chooser_get_preview_filename (file_chooser);
	GdkPixbuf *pixbuf = NULL;
	if (filename != NULL)
	{
		pixbuf = gdk_pixbuf_new_from_file_at_size (filename, 64, 64, NULL);
		g_free (filename);
	}
	if (pixbuf != NULL)
	{
		gtk_image_set_from_pixbuf (GTK_IMAGE (preview), pixbuf);
		gdk_pixbuf_unref (pixbuf);
		gtk_file_chooser_set_preview_widget_active (file_chooser, TRUE);
	} else
		gtk_file_chooser_set_preview_widget_active (file_chooser, FALSE);
}

  /*****************/
 /***** utils *****/
/*****************/

static GtkTreeModel *create_model (GtkWidget *dialog)
{
	GtkListStore *store = gtk_list_store_new (1, G_TYPE_STRING);

	GSList *ids = gtk_stock_list_ids ();
	ids = g_slist_sort (ids, (GCompareFunc) strcmp);
	GSList *tmp_list = ids;
	while (tmp_list != NULL)
	{
		GtkTreeIter iter;
		gtk_list_store_append (store, &iter);
		gtk_list_store_set (store, &iter, 0, tmp_list->data, -1);
		tmp_list = g_slist_next (tmp_list);
	}
	
	g_slist_foreach (ids, (GFunc) g_free, NULL);
	g_slist_free (ids);

	return GTK_TREE_MODEL (store);
}

static void cell_data_func (GtkTreeViewColumn *tree_column,
	GtkCellRenderer *cell, GtkTreeModel *model, GtkTreeIter *iter, gpointer data)
{
	gchar *value;
	gtk_tree_model_get (model, iter, 0, &value, -1);
	if (g_str_has_prefix (value, "gtk-"))
		value += 4;
	g_object_set (GTK_CELL_RENDERER (cell), "text", value, NULL);
}


static GtkWidget *create_stock_icon_browser (E2_SID_Runtime *rt)
{
	GtkWidget *sw = e2_widget_get_sw (GTK_POLICY_NEVER,
		GTK_POLICY_AUTOMATIC);

	rt->model = create_model (sw);
	rt->treeview = GTK_TREE_VIEW (gtk_tree_view_new_with_model (rt->model));
	gtk_tree_view_set_rules_hint (rt->treeview, TRUE);

	g_object_unref (rt->model);

	g_signal_connect (rt->treeview, "row-activated", G_CALLBACK (activated_cb),
		rt->dialog);
	
	gtk_container_add (GTK_CONTAINER (sw), GTK_WIDGET (rt->treeview));
	GtkTreeViewColumn *column = gtk_tree_view_column_new ();
	g_object_set (column, "alignment", 0.5, NULL);
	gtk_tree_view_column_set_title (column, "icons");

	GtkCellRenderer *cell_renderer = gtk_cell_renderer_pixbuf_new ();
	gtk_tree_view_column_pack_start (column, cell_renderer, FALSE);
	gtk_tree_view_column_set_attributes (column, cell_renderer, "stock_id", 0,
		NULL);
	g_object_set (cell_renderer, "xpad", 10, NULL);
	cell_renderer = gtk_cell_renderer_text_new ();
	gtk_tree_view_column_pack_start (column, cell_renderer, FALSE);
	gtk_tree_view_column_set_cell_data_func (column, cell_renderer,
		cell_data_func, NULL, NULL);
	g_object_set (cell_renderer, "xpad", 10, NULL);
	gtk_tree_view_append_column (rt->treeview, column);

	return sw;
}

static GtkWidget *create_file_chooser (GtkWidget *dialog)
{
	chdir ("/");
	GtkWidget *file_chooser = gtk_file_chooser_widget_new
		(GTK_FILE_CHOOSER_ACTION_OPEN);
	gtk_file_chooser_set_local_only (GTK_FILE_CHOOSER (file_chooser), TRUE);
	gtk_file_chooser_add_shortcut_folder (GTK_FILE_CHOOSER (file_chooser),
		ICON_DIR, NULL);
//	gtk_file_chooser_add_shortcut_folder (GTK_FILE_CHOOSER (file_chooser),
//		app.pane1.path, NULL);
//	gtk_file_chooser_add_shortcut_folder (GTK_FILE_CHOOSER (file_chooser),
//		app.pane2.path, NULL);
	
	//add filters
	GtkFileFilter *filter = gtk_file_filter_new ();
	gtk_file_filter_set_name (GTK_FILE_FILTER (filter), _("images"));
	gtk_file_filter_add_mime_type (filter, "image/png");
	gtk_file_filter_add_mime_type (filter, "image/x-xpixmap");
	gtk_file_filter_add_mime_type (filter, "image/x-xpm");
	gtk_file_filter_add_mime_type (filter, "image/svg+xml");
	//FIXME: check the mime types
	gtk_file_filter_add_mime_type (filter, "image/jpg");
	gtk_file_filter_add_mime_type (filter, "image/bmp");
	gtk_file_filter_add_mime_type (filter, "image/gif");
	gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (file_chooser), filter);
	filter = gtk_file_filter_new ();
	gtk_file_filter_set_name (GTK_FILE_FILTER (filter), _("all"));
	gtk_file_filter_add_pattern (filter, "*");
	gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (file_chooser), filter);
	
	GtkWidget *preview = gtk_image_new ();
	gtk_file_chooser_set_preview_widget (GTK_FILE_CHOOSER (file_chooser),
		preview);
	g_signal_connect (file_chooser, "update-preview",
		G_CALLBACK (update_preview_cb), preview);

	gtk_file_chooser_set_action (GTK_FILE_CHOOSER (file_chooser),
		GTK_FILE_CHOOSER_ACTION_OPEN);

	//FIXME: readd missing location entry to the dialog
//	GtkWidget *entry = gtk_entry_new ();
//	gtk_file_chooser_set_extra_widget (GTK_FILE_CHOOSER (file_chooser), entry);
	
	g_signal_connect (file_chooser, "file-activated",
		G_CALLBACK (file_activated_cb), dialog);

	return file_chooser;
}

static void setup_dialog (E2_SID_Runtime *rt)
{
	GtkStockItem item;
	if (gtk_stock_lookup (rt->icon, &item))
	{
		gtk_notebook_set_current_page (rt->notebook, 1);
		rt->page = 1;
		GtkTreeIter iter;
		if (gtk_tree_model_get_iter_first (rt->model, &iter))
			do
			{
				gchar *value;
				gtk_tree_model_get (rt->model, &iter, 0, &value, -1);
				if (g_str_equal (rt->icon, value))
				{
					GtkTreePath *path = gtk_tree_model_get_path (rt->model, &iter);
					if (path != NULL)
					{
						GtkTreeViewColumn *column = gtk_tree_view_get_column (rt->treeview, 0);
						gtk_tree_view_set_cursor (rt->treeview, path, column, FALSE);
						gtk_tree_view_scroll_to_cell (rt->treeview, path, column, TRUE, 0.5, 0.5);
					}
					break;
				}
			} while (gtk_tree_model_iter_next (rt->model, &iter));
		gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (rt->file_chooser),
			ICON_DIR);
	} else
	{
		gtk_notebook_set_current_page (rt->notebook, 0);
		rt->page = 0;
		if ((rt->icon != NULL) && (*rt->icon != '\0'))
			gtk_file_chooser_set_filename (GTK_FILE_CHOOSER (rt->file_chooser),
				rt->icon);
		else
			gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (rt->file_chooser),
				ICON_DIR);
	}
}

  /******************/
 /***** public *****/
/******************/

GtkWidget *e2_sid_create (GtkWidget *parent, const gchar *name,
	gchar *icon, gpointer response_func, gpointer data)
{
	//get or create and inititialize select image dialog runtime object and
	//ensure that there exists only one runtime object (and only one dialog)
	//for every parent widget
	E2_SID_Runtime *rt =  g_object_get_data (G_OBJECT (parent), name);
	//the dialog already exists
	if (rt != NULL)
	{
		printd (NOTICE, "select image dialog already exists");
		rt->icon = icon;
		//set it up for the possibly new icon,
		setup_dialog (rt);
		//present the window
		gtk_window_present (GTK_WINDOW (rt->dialog));
		//and return
		return rt->dialog;
	} else
	{
		printd (NOTICE, "creating select image dialog");
		rt = g_new0 (E2_SID_Runtime, 1);
		//set up runtime object
		rt->name = g_strdup (name);
		rt->icon = g_strdup (icon);
		rt->parent = parent;
	}
	
	//
	//create dialog widgets
	//

	//the local response_cb response func sets up object properties so it is
	//possible to determine the currently selected icon or file
	rt->dialog = e2_dialog_create (NULL, NULL, rt->name, response_cb, rt);
	//the supplied response func might update the widget the select image
	//dialog was used for
	g_signal_connect (rt->dialog, "response", G_CALLBACK (response_func), data);
	//ensure the dialog is destroyed with the parent widget
	if (parent != NULL)
		g_object_set_data_full (G_OBJECT (parent), rt->name, rt,
			(GDestroyNotify) destroy_cb2);
	//when the dialog is destroyed, free the runtime object
	g_signal_connect (rt->dialog, "destroy", G_CALLBACK (destroy_cb), rt);

	//a notebook is used to change between the stock icon selector and the
	//file chooser
	rt->notebook = GTK_NOTEBOOK (e2_widget_add_notebook
		(GTK_DIALOG (rt->dialog)->vbox, TRUE, 5, switch_page_cb, rt));

	//the new gtk file chooser is used
	rt->file_chooser = create_file_chooser (rt->dialog);
	//the file chooser is inside a scrolled window bcause the availabel size
	//might change
	GtkWidget *sw = e2_widget_get_sw (GTK_POLICY_NEVER, GTK_POLICY_NEVER);
	e2_widget_sw_add_with_viewport (sw, rt->file_chooser);
	GtkWidget *label = gtk_label_new_with_mnemonic ("_file selection");
	//add the scrolled window to the notebook
	gtk_notebook_append_page_menu (rt->notebook, sw, label, label);
	//and the stock icon browser
	GtkWidget *stock = create_stock_icon_browser (rt);
	label = gtk_label_new_with_mnemonic ("_stock icons");
	gtk_notebook_append_page_menu (rt->notebook, stock, label, label);

	//add dialog buttons
	e2_dialog_add_button (rt->dialog, E2_BUTTON_CANCEL | E2_BUTTON_REMOVE |
		E2_BUTTON_APPLY | E2_BUTTON_OK);
	e2_dialog_show (rt->dialog, rt->parent, 0, 0);
	//show the currently selected icon/file
	setup_dialog (rt);
	
	return rt->dialog;
}

