/* $Id: e2_keybinding.c,v 1.14 2004/06/08 23:47:34 cairo Exp $
 * 
 * Copyright (C) 2004 tooar <tooar@gmx.net>
 * 
 * This file is part of emelfm2.
 * emelfm2 is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * emelfm2 is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with emelFM2; see the file COPYING. If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdlib.h>
#include <stdio.h>
#include "emelfm2.h"
#include "e2_keybinding.h"

E2_OptionSet *opt_keys;
GList *keys;
GList *keys_cur = NULL;

  /*****************/
 /***** utils *****/
/*****************/

static void _free_keys (GList *keys)
{
	GList *list;
	for (list = keys; list != NULL; list = g_list_next (list))
	{
		E2_KeyRuntime *k = list->data;
		if (k != NULL)
		{
			if (k->chained != NULL)
				_free_keys (k->chained);
			g_free (k->action);
			g_free (k->action_data);
			g_free (k);
		}
	}
	g_list_free (keys);
}

static void free_keys (E2_KeybindingRuntime *rt)
{
	if (rt->keys != NULL)
	{
		_free_keys (rt->keys);
		rt->keys = NULL;
	}
}

static void adjust_buttons (GtkTreeView *view, gboolean value)
{
	E2_Quintet *q = g_object_get_data (G_OBJECT (view), "e2-config-buttons");
	if (q == NULL)
		return;
	if (q->a != NULL)
		gtk_widget_set_sensitive (q->a, value);
	if (q->b != NULL)
		gtk_widget_set_sensitive (q->b, value);
	if (q->c != NULL)
		gtk_widget_set_sensitive (q->c, value);
	if (q->d != NULL)
		gtk_widget_set_sensitive (q->d, value);
}

static void _sync_keybinding (GtkTreeModel *model, GtkTreeIter *iter, GList **keys)
{
	gchar *cat, *key, *action, *action_data;
	gboolean cont;
	gtk_tree_model_get (model, iter, 0, &cat, 1, &key, 2, &cont, 3 , &action,
		4, &action_data, -1);
	if (((cat != NULL) && (cat[0] != '\0')) || (key == NULL))
	{
		g_free (cat);
		g_free (key);
		g_free (action);
		g_free (action_data);
		return;
	}
	GdkModifierType mask;
	guint keyval;
	gtk_accelerator_parse (key, &keyval, &mask);
	if ((keyval != 0) || (mask != 0))
	{
//		printd (DEBUG, "key - %s - %d - %d", key, keyval, mask);
		E2_KeyRuntime *k = g_malloc (sizeof (E2_KeyRuntime));
		k->keyval = keyval;
		k->mask = mask;
		k->action = action;
		k->action_data = action_data;
		k->cont = cont;
		k->chained = NULL;
		GtkTreePath *path = gtk_tree_model_get_path (model, iter);
		gtk_tree_path_down (path);
		GtkTreeIter child_iter;
		while (gtk_tree_model_get_iter (model, &child_iter, path))
		{
			_sync_keybinding (model, &child_iter, &k->chained);
			gtk_tree_path_next (path);
		}
		gtk_tree_path_free (path);
		*keys = g_list_append (*keys, k);
	} else
	{
		g_free (action);
		g_free (action_data);
	}
	g_free (cat);
	g_free (key);
	return;
}

static gboolean sync_keybinding (E2_KeybindingRuntime *rt)
{
	if (!gtk_tree_row_reference_valid (rt->ref))
	{
		printd (WARN, "internal keybinding error, row reference not valid anymore");
		return FALSE;
	}
	GtkTreePath *path = gtk_tree_row_reference_get_path (rt->ref);
	if (path == NULL)
	{
		printd (WARN, "internal keybinding error, path is NULL");
		return FALSE;
	}

	free_keys (rt);

	gtk_tree_path_down (path);
	GtkTreeIter iter;
	while (gtk_tree_model_get_iter (rt->model, &iter, path))
	{
		_sync_keybinding (rt->model, &iter, &rt->keys);
		gtk_tree_path_next (path);
	}
	gtk_tree_path_free (path);
	return TRUE;
}

static GList *find_by_name (const gchar *name)
{
	GList *list = keys;
	while (list != NULL)
	{
		if (g_str_equal (name, ((E2_KeybindingRuntime *) list->data)->name))
			return list;
		list = g_list_next (list);
	}
	return NULL;
}

  /*********************/
 /***** callbacks *****/
/*********************/

static gboolean remove_chained_idle_cb (E2_KeybindingRuntime *rt)
{
	rt->chained = NULL;
	rt->timeout_id = 0;
	return FALSE;
}

gboolean tree_selection_check_cb (GtkTreeSelection *selection,
	GtkTreeModel *model, GtkTreePath *path,
	gboolean path_currently_selected, E2_OptionSet *set)
{
	GtkTreeIter iter;
	if (gtk_tree_model_get_iter (set->ex.tree.model, &iter, path))
	{
		gchar *cat;
		gtk_tree_model_get (set->ex.tree.model, &iter, 0, &cat, -1);
		if (cat[0] != '\0')
		{
			GtkTreeView *view = gtk_tree_selection_get_tree_view (selection);
			adjust_buttons (view, path_currently_selected);
		}
		g_free (cat);
	}
	return TRUE;
}

static gboolean visible_check_cb (GtkTreeModel *model,
	GtkTreeIter *iter, GtkCellRenderer *cell, gpointer data)
{
	gchar *cat;
	gtk_tree_model_get (model, iter, 0, &cat, -1);
	gboolean retval = (cat[0] == '\0' ? TRUE : FALSE);
	g_free (cat);
	return retval;
}

static gboolean tree_draggable_check_cb (GtkTreeDragSource *drag_source,
	GtkTreePath *path)
{
	if (!GTK_IS_TREE_MODEL (drag_source))
		return TRUE;
	GtkTreeModel *model = GTK_TREE_MODEL (drag_source);
	GtkTreeIter iter;
	if (gtk_tree_model_get_iter (model, &iter, path))
	{
		gchar *cat;
		gtk_tree_model_get (model, &iter, 0, &cat, -1);
		if (cat[0] == '\0')
		{
			g_free (cat);
			return TRUE;
		}
		g_free (cat);
		return FALSE;
	}
	return TRUE;
}

static gboolean key_press_cb (GtkWidget *widget, GdkEventKey *event,
	E2_KeybindingRuntime *rt)
{
	guint modifiers = gtk_accelerator_get_default_mod_mask ();
	modifiers &= event->state;
	//printd (DEBUG, "key 1: %d - %d", event->keyval, modifiers);
	GList *list;
	for (list = rt->chained; list != NULL; list = g_list_next (list))
	{
		E2_KeyRuntime *k = list->data;
		if ((k->keyval == event->keyval) && (k->mask == modifiers))
		{
			if (k->chained != NULL)
			{
				rt->chained = k->chained;
				if (rt->timeout_id != 0)
					g_source_remove (rt->timeout_id);
				rt->timeout_id = g_timeout_add (e2_option_int_get ("keybindings-timeout"),
					(GSourceFunc) remove_chained_idle_cb, rt);
			} else
			{
				if (rt->timeout_id != 0)
					g_source_remove (rt->timeout_id);
				remove_chained_idle_cb (rt);
			}
				
			e2_action_run_simple_from (k->action, k->action_data, widget);
			if (!k->cont)
				return TRUE;
		}
	}
	for (list = rt->keys; list != NULL; list = g_list_next (list))
	{
		E2_KeyRuntime *k = list->data;
		if ((k->keyval == event->keyval) && (k->mask == modifiers))
		{
			if (k->chained != NULL)
			{
				rt->chained = k->chained;
				if (rt->timeout_id != 0)
					g_source_remove (rt->timeout_id);
				rt->timeout_id = g_timeout_add (e2_option_int_get ("keybindings-timeout"),
					(GSourceFunc) remove_chained_idle_cb, rt);
			}
			e2_action_run_simple_from (k->action, k->action_data, widget);
			if (!k->cont)
				return TRUE;
		}
	}
	return FALSE;
}

static gboolean idle_sync_cb (gpointer data)
{
	if (keys_cur == NULL)
		keys_cur = keys;
	if (keys_cur == NULL)
		return FALSE;
	else
	{
		//sync a single KeybindingRuntime
		gint retval = sync_keybinding ((E2_KeybindingRuntime *) keys_cur->data);
		keys_cur = g_list_next (keys_cur);
		if (keys_cur != NULL)
			return retval;
	}
	return FALSE;
}

static void model_changed_cb ()
{
	keys_cur = NULL;
	g_idle_add ((GSourceFunc) idle_sync_cb, NULL);
}

//static void widget_destroyed_cb (E2_KeybindingRuntime *rt)
//{
//}

  /******************/
 /***** public *****/
/******************/

void e2_keybinding_register (const gchar *category, GtkWidget *widget)
{
	//check if it already exists
	GList *list = find_by_name (category);
	if (list != NULL)
	{
		printd (NOTICE, "keybinding '%s' already exists", category);
		E2_KeybindingRuntime *rt = list->data;
		(rt->ref_count)++;
		g_signal_connect (G_OBJECT (widget), "key-press-event",
			G_CALLBACK (key_press_cb), rt);
		return;
	}

	//check if there is a parent category
	gchar *_parent = g_utf8_strrchr (category, -1, '.');
	GtkTreeIter iter_parent;
	if (_parent != NULL)
	{
		gchar *parent = g_strndup (category, _parent - category);
		GList *plist = find_by_name (parent);
		E2_KeybindingRuntime *rt = plist->data;
		GtkTreePath *path = gtk_tree_row_reference_get_path (rt->ref);
		if (path != NULL)
			gtk_tree_model_get_iter (opt_keys->ex.tree.model, &iter_parent, path);
		g_free (parent);
		gtk_tree_path_free (path);
	} else
		gtk_tree_model_get_iter_first (opt_keys->ex.tree.model, &iter_parent);
	const gchar *name = (((_parent != NULL) && (_parent[0] != '\0')) ?
		_parent + 1 : category);
	GtkTreeIter iter = iter_parent;
	if (!_e2_tree_find_iter_from_str (opt_keys->ex.tree.model, 0, name, &iter, FALSE))
	{
		gpointer options[1] = { (gpointer) name };
		_e2_option_tree_add (opt_keys, &iter, _parent != NULL ? &iter_parent : NULL,
			FALSE, TRUE, 1, options);
	}
	GtkTreePath *path = gtk_tree_model_get_path (opt_keys->ex.tree.model, &iter);

	//init runtime object
	E2_KeybindingRuntime *rt = g_malloc (sizeof (E2_KeybindingRuntime));
	rt->name = g_strdup (category);
	rt->model = opt_keys->ex.tree.model;
	rt->ref = gtk_tree_row_reference_new (opt_keys->ex.tree.model, path);
	rt->keys = NULL;
	rt->ref_count = 2;
	rt->chained = NULL;
	rt->timeout_id = 0;
//	g_object_set_data_full (G_OBJECT (widget), "e2-keybinding-runtime",
//		rt, (GDestroyNotify) widget_destroyed_cb);
	g_signal_connect (G_OBJECT (widget), "key-press-event",
		G_CALLBACK (key_press_cb), rt);
	g_signal_connect (G_OBJECT (rt->model), "row-changed",
		G_CALLBACK (model_changed_cb), NULL);
	keys = g_list_append (keys, rt);
	g_idle_add ((GSourceFunc) idle_sync_cb, NULL);
	//cleanup
	gtk_tree_path_free (path);
}

void e2_keybinding_clean ()
{
//	GList *list;
/*	for (list = keys; list != NULL; list = g_list_next (list))
	{
		free_keys (rt);
		gtk_tree_row_reference_free (rt->ref);
		g_free (rt->name);
		g_free (rt); 
	} */
	g_list_free (keys);
	keys = NULL;
	keys_cur = NULL;
}

void e2_keybinding_init ()
{
	keys = NULL;
	opt_keys = e2_option_tree_register ("keybindings", "keybindings",
		"keybindings", NULL, tree_selection_check_cb, tree_draggable_check_cb,
		E2_OPTION_TREE_UP_DOWN | E2_OPTION_TREE_ADD_DEL);
	e2_option_tree_add_column (opt_keys, "category", E2_OPTION_TREE_TYPE_STR, 0, "",
		NULL, NULL, E2_OPTION_TREE_SET_NOT_EDITABLE, NULL, NULL);
	e2_option_tree_add_column (opt_keys, "key", E2_OPTION_TREE_TYPE_KEY, 0, "", NULL,
		NULL, 0, visible_check_cb, NULL);
	e2_option_tree_add_column (opt_keys, "continue", E2_OPTION_TREE_TYPE_BOOL,
		FALSE, "false", NULL, NULL, 0, visible_check_cb, NULL);
	e2_option_tree_add_column (opt_keys, "action", E2_OPTION_TREE_TYPE_SEL,
		0, "<custom command>", e2_actions_fill_store,
		(gpointer) (E2_ACTION_EXCLUDE_ACCEL), 0, visible_check_cb, NULL);
	e2_option_tree_add_column (opt_keys, "argument", E2_OPTION_TREE_TYPE_STR, 0, "",
		NULL, NULL, 0, visible_check_cb, NULL);
	e2_option_tree_sync_columns (opt_keys);
	_e2_option_tree_append_default (opt_keys,
		"<keybindings",
		"main||false||",
		"	|<Ctrl>space|false|output.print|directory jump keys activated\\n",
		"		|h|false|<custom command>|cd $(HOME)",
		"		|m|false|<custom command>|cd /mnt/",
		"		|d|false|<custom command>|cd $(HOME)/downloads",
		"	|F1|false|panes.refresh|",
		"	|F2|false|file.rename|",
		"	|F3|false|find|",
		"	|F4|false|view|",
		"	|F5|false|file.copy|",
		"	|F6|false|file.move|",
		"	|F7|false|dialog.mkdir|",
		"	|F8|false|file.delete|",
		"	|F9|false|file.file info|",
		"	|F10|false|quit|",
		"	|F11|false|config.configure|",
		"	|F12|false|panes.toggle direction|",
		"	|<Control>w|false|output.adjust ratio|*,100%",
		"	|<Control>e|false|output.adjust ratio|*,0",
		"	|<Control>z|false|command line.focus toggle|",
		"	|<Control>r|false|panes.refresh|",
		"	file pane||false||",
		"		|Tab|false|panes.switch|",
		"		|space|false|panes.switch|",
		"		|Return|false|file.open|",
		"		|<Shift>Return|false|file.open in other|",
		"		|Left|false|<custom command>|cd ..",
		"		|Right|false|file.open|",
		"		|<Shift>Left|false|panes.go back|",
		"		|<Shift>Right|false|panes.go forward|",
		"		|BackSpace|false|<custom command>|cd ..",
		"		|Delete|false|file.delete|",
		"		|Insert|false|file.toggle tag|",
		"		|<Control>t|false|file.toggle tag|",
		"		|Home|false|<custom command>|cd $(HOME)",
		"		|<Control>a|false|panes.select all|",
		"		|<Control>f|false|find|",
		"		|<Control>x|false|panes.sync|",
		"		|<Control>h|false|panes.toggle hidden|",
		"		|slash|false|find|",
		"		|comma|false|panes.adjust ratio|*,100%",
		"		|period|false|panes.adjust ratio|*,0",
		"		|<Control>p|false|panes.show menu|",
		"		|<Control>parenleft|false|panes.show menu|shift",
		"		|<Control>parenright|false|panes.show menu|ctrl",
		"	dir-line||false||",
		"		|Tab|false|command line.complete|dirs",
		"		|Insert|false|command line.clear|",
		"		|<Control>Return|false|command line.insert selection|escape",
		"		|<Alt>Return|false|command line.insert selection|",
		"	command-line||false||",
		"		|<Control>Return|false|command line.insert selection|escape",
		"		|<Alt>Return|false|command line.insert selection|quote",
		"		|Insert|false|command line.clear|",
		"		|Tab|false|command line.complete|",
		"		|Page_Up|false|output.page up|",
		"		|Page_Down|false|output.page down|",
		"		|<Shift>Page_Up|false|output.goto top|",
		"		|<Shift>Page_Down|false|output.goto bottom|",
		"		|<Shift>Up|false|output.scroll up|",
		"		|<Shift>Down|false|output.scroll down|",
		">", NULL);
		
	e2_option_int_register ("keybindings-timeout", "interface:delays",
		_("chained keybindings timeout (ms)"), _("This option sets the number of "
		"milliseconds to wait before deactivating a chained subset of keybindings."),
		NULL, 1000, 1, 1000000);

}
