/* $Id: e2_toolbar.c,v 1.43 2004/07/08 00:31:57 cairo Exp $
 * 
 * Copyright (C) 2003-2004 tooar <tooar@gmx.net>
 * 
 * This file is part of emelFM2.
 * emelFM2 is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * emelFM2 is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with emelFM2; see the file COPYING. If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include "emelfm2.h"
#include "e2_toolbar.h"
#include "e2_option.h"
#include "debug.h"
#include "e2_context_menu.h"
#include "e2_action.h"
#include "e2_utils.h"
#include "e2_button.h"
#include "e2_bookmark.h"
#include "e2_tree.h"

static void resize_cb (GtkWidget *handlebox, GtkAllocation *alloc, E2_ToolbarRuntime *rt);
static GtkWidget *toolbar_add_button (E2_ToolbarRuntime *rt, gchar *name,
	gchar *icon, gchar *tip, gpointer data);

  /*******************/
 /***** actions *****/
/*******************/

static void toggle_toolbar_cb (E2_ToolbarRuntime *rt)
{
	if (_e2_option_bool_toggle (rt->show))
		_e2_toolbar_create (rt);
	else
		e2_toolbar_destroy (rt);
}

  /*********************/
 /***** callbacks *****/
/*********************/

static void edit_cb (GtkWidget *menuitem, E2_ToolbarRuntime *rt)
{
	create_config_dialog (rt->set->desc);
}

static void toggle_tb_cb (GtkWidget *menuitem, E2_ToolbarRuntime *rt)
{
	if (!rt->blocked)
	{
		_e2_option_bool_toggle (rt->show);
		e2_toolbar_destroy (rt);
		_e2_toolbar_recreate (rt);
	}
}

static void update_tt_cb (GtkWidget *menuitem, E2_ToolbarRuntime *rt)
{
	if (!rt->blocked)
	{
		gtk_toolbar_set_tooltips (GTK_TOOLBAR (rt->toolbar),
			_e2_option_bool_toggle (rt->tooltips));
	}
}

static void update_space_cb (GtkWidget *menuitem, E2_ToolbarRuntime *rt)
{
	if (!rt->blocked)
	{
		GSList *group = gtk_radio_menu_item_get_group (GTK_RADIO_MENU_ITEM (menuitem));
		gint index = 2 - g_slist_index (group, menuitem);
		if (_e2_option_int_get (rt->space) != index)
		{
			_e2_option_sel_set (rt->space, index);
			_e2_toolbar_recreate (rt);
		}
	}
}

static void update_hori_cb (GtkWidget *menuitem, E2_ToolbarRuntime *rt)
{
	if (!rt->blocked)
	{
		_e2_option_bool_toggle (rt->hori);
		_e2_toolbar_recreate (rt);
	}
}

static void update_type_cb (GtkWidget *menuitem, E2_ToolbarRuntime *rt)
{
	if (!rt->blocked)
	{
		GSList *group = gtk_radio_menu_item_get_group (GTK_RADIO_MENU_ITEM (menuitem));
		gint index = 3 - g_slist_index (group, menuitem);
		if (_e2_option_int_get (rt->type) != index)
		{
			_e2_option_sel_set (rt->type, index);
			_e2_toolbar_recreate (rt);
		}
	}
}

static void increase_priority_cb (GtkWidget *menuitem, E2_ToolbarRuntime *rt)
{
	if (!rt->blocked)
	{
		gint i = _e2_option_int_get (rt->priority);
		if (_e2_option_int_set (rt->priority, i + 1) != i)
			_e2_toolbar_recreate (rt);
	}
}

static void decrease_priority_cb (GtkWidget *menuitem, E2_ToolbarRuntime *rt)
{
	if (!rt->blocked)
	{
		gint i = _e2_option_int_get (rt->priority);
		if (_e2_option_int_set (rt->priority, i - 1) != i)
			_e2_toolbar_recreate (rt);
	}
}

static void update_relief_cb (GtkWidget *menuitem, E2_ToolbarRuntime *rt)
{
	if (!rt->blocked)
	{
		_e2_option_bool_toggle (rt->relief);
		_e2_toolbar_recreate (rt);
	}
}

static void update_same_cb (GtkWidget *menuitem, E2_ToolbarRuntime *rt)
{
	if (!rt->blocked)
	{
		_e2_option_bool_toggle (rt->same);
		_e2_toolbar_recreate (rt);
	}
}

static void reset_priority_cb (GtkWidget *menuitem, E2_ToolbarRuntime *rt)
{
	if (!rt->blocked && (_e2_option_int_get (rt->priority) != 0))
	{
		_e2_option_int_set (rt->priority, 0);
		_e2_toolbar_recreate (rt);
	}
}

static void update_style_cb (GtkWidget *menuitem, E2_ToolbarRuntime *rt)
{
	if (!rt->blocked)
	{
		GSList *group = gtk_radio_menu_item_get_group (GTK_RADIO_MENU_ITEM (menuitem));
		gint index = 3 - g_slist_index (group, menuitem);
		if (_e2_option_int_get (rt->style) != index)
		{
			_e2_option_sel_set (rt->style, index);
			_e2_toolbar_recreate (rt);
		}
	}
}

static void update_isize_cb (GtkWidget *menuitem, E2_ToolbarRuntime *rt)
{
	if (!rt->blocked)
	{
		GSList *group = gtk_radio_menu_item_get_group (GTK_RADIO_MENU_ITEM (menuitem));
		gint index = 5 - g_slist_index (group, menuitem);
		if (_e2_option_int_get (rt->isize) != index)
		{
			_e2_option_sel_set (rt->isize, index);
			_e2_toolbar_recreate (rt);
		}
	}
}

static gboolean click_cb (GtkWidget *widget, GdkEventButton *event, E2_ToolbarRuntime *rt)
{
	if ((event->button == 3) && (event->type == GDK_BUTTON_PRESS))
	{
		//block local callbacks
		rt->blocked = TRUE;
		GtkWidget *menu, *submenu, *menuitem;
		menu = gtk_menu_new ();
		menuitem = add_menu_item2 (menu, "_edit bar", GTK_STOCK_PROPERTIES, edit_cb, rt);
		e2_menu_add_separator (menu);
		menuitem = e2_menu_add_check (menu, "sho_w bar",
			_e2_option_bool_get (rt->show), toggle_tb_cb, rt);
		menuitem = e2_menu_add_check (menu, "show toolti_ps",
			_e2_option_bool_get (rt->tooltips), update_tt_cb, rt);
		submenu = add_submenu2 (menu, "_space handling");
		GSList *group = NULL;
		menuitem = add_menu_radio_button (submenu, &group, "_none",
			(_e2_option_int_get (rt->space) == 0), update_space_cb, rt);
		menuitem = add_menu_radio_button (submenu, &group, "_scrollbars",
			(_e2_option_int_get (rt->space) == 1), update_space_cb, rt);
		menuitem = add_menu_radio_button (submenu, &group, "_rest button",
			(_e2_option_int_get (rt->space) == 2), update_space_cb, rt);
		e2_menu_add_separator (menu);
		submenu = add_submenu2 (menu, "_type");
		group = NULL;
		menuitem = add_menu_radio_button (submenu, &group, "_main toolbar",
			(_e2_option_int_get (rt->type) == 0), update_type_cb, rt);
		menuitem = add_menu_radio_button (submenu, &group, "_big panebar",
			(_e2_option_int_get (rt->type) == 1), update_type_cb, rt);
		menuitem = add_menu_radio_button (submenu, &group, "_small panebar 1",
			(_e2_option_int_get (rt->type) == 2), update_type_cb, rt);
		menuitem = add_menu_radio_button (submenu, &group, "_small panebar 2",
			(_e2_option_int_get (rt->type) == 3), update_type_cb, rt);
		submenu = add_submenu2 (menu, "_position");
		menuitem = e2_menu_add_check (submenu, "_horizontal",
			_e2_option_bool_get (rt->hori), update_hori_cb, rt);
		if (_e2_option_bool_get (rt->hori))
		{
			menuitem = add_menu_item2 (submenu, "_up (decrease proirity)", GTK_STOCK_GO_UP, decrease_priority_cb, rt);
			menuitem = add_menu_item2 (submenu, "_down (increase priority)", GTK_STOCK_GO_DOWN, increase_priority_cb, rt);
		} else
		{
			menuitem = add_menu_item2 (submenu, "_left (decrease priority)", GTK_STOCK_GO_BACK, decrease_priority_cb, rt);
			menuitem = add_menu_item2 (submenu, "_right (increase priority)", GTK_STOCK_GO_FORWARD, increase_priority_cb, rt);
		}
		menuitem = add_menu_item2 (submenu, "_reset priority", NULL, reset_priority_cb, rt);
		e2_menu_add_separator (menu);
		menuitem = e2_menu_add_check (menu, "buttons relief",
			_e2_option_bool_get (rt->relief), update_relief_cb, rt);
		menuitem = e2_menu_add_check (menu, "buttons same size",
			_e2_option_bool_get (rt->same), update_same_cb, rt);
		submenu = add_submenu2 (menu, "_button style");
		group = NULL;
		menuitem = add_menu_radio_button (submenu, &group, "_icons only",
			(_e2_option_int_get (rt->style) == 0), update_style_cb, rt);
		menuitem = add_menu_radio_button (submenu, &group, "_text only",
			(_e2_option_int_get (rt->style) == 1), update_style_cb, rt);
		menuitem = add_menu_radio_button (submenu, &group, "_icons above text",
			(_e2_option_int_get (rt->style) == 2), update_style_cb, rt);
		menuitem = add_menu_radio_button (submenu, &group, "_icons aside text",
			(_e2_option_int_get (rt->style) == 3), update_style_cb, rt);
		submenu = add_submenu2 (menu, "_icon size");
		group = NULL;
		menuitem = add_menu_radio_button (submenu, &group, "_menu",
			(_e2_option_int_get (rt->isize) == 0), update_isize_cb, rt);
		menuitem = add_menu_radio_button (submenu, &group, "toolbar _small",
			(_e2_option_int_get (rt->isize) == 1), update_isize_cb, rt);
		menuitem = add_menu_radio_button (submenu, &group, "toolbar _large",
			(_e2_option_int_get (rt->isize) == 2), update_isize_cb, rt);
		menuitem = add_menu_radio_button (submenu, &group, "button",
			(_e2_option_int_get (rt->isize) == 3), update_isize_cb, rt);
		menuitem = add_menu_radio_button (submenu, &group, "drag 'n' drop",
			(_e2_option_int_get (rt->isize) == 4), update_isize_cb, rt);
		menuitem = add_menu_radio_button (submenu, &group, "dialog",
			(_e2_option_int_get (rt->isize) == 5), update_isize_cb, rt);

		e2_menu_popup (menu, event->button, event->time);
		rt->blocked = FALSE;
		return TRUE;
	}
	return FALSE;
}

static void menu_destroy_cb (GtkWidget *menu_shell, GtkWidget *menu)
{
	gtk_container_foreach (GTK_CONTAINER (menu), (void *) gtk_widget_destroy, NULL);
	gtk_widget_destroy (menu);
}

static GtkWidget *get_rest_menu (E2_ToolbarRuntime *rt)
{
	GList *last = g_list_last (rt->buttons_hidden);
	GtkTreePath *path = g_object_get_data (G_OBJECT (last->data), "path");
	while ((path == NULL) && ((last = g_list_previous (last)) != NULL))
		path = g_object_get_data (G_OBJECT (last->data), "path");
	if (path != NULL)
	{
		GtkTreeIter iter;
		if (gtk_tree_model_get_iter (rt->set->ex.tree.model, &iter, path))
		{
			GtkWidget *menu = gtk_menu_new ();
			e2_toolbar_add_menu_item (menu,
				rt->set->ex.tree.model, &iter, 1, rt);
			g_signal_connect (G_OBJECT (menu), "selection-done",
				G_CALLBACK (menu_destroy_cb), menu);
			return menu;
		}
	}
	return NULL;
}

static void tb_menu_position (GtkMenu *menu, gint *x, gint *y,
	gboolean *push_in, GtkAllocation *menu_position)
{
	*x = menu_position->x;
	*y = menu_position->y;
	*push_in = TRUE;
}

static gboolean menu_popup (GtkWidget *item, gint x, gint y, E2_ToolbarRuntime *rt,
	gint button, guint32 time)
{
	GtkWidget *menu = g_object_get_data (G_OBJECT (item), "menu");
	if (menu == NULL)
	{
		if (rt != NULL)
			menu = get_rest_menu (rt);
		else
			return FALSE;
	}
	GtkRequisition menu_size;
	gtk_widget_size_request (menu, &menu_size);
	gint parent_width;
	if (_e2_option_bool_get (rt->hori))
		parent_width = item->requisition.width;
	else
		parent_width = GTK_WIDGET (rt->toolbar)->requisition.width;
	if (parent_width > menu_size.width)
		gtk_widget_set_size_request (menu, parent_width, -1);
	gint screen_height = gdk_screen_height ();
	if (y + menu_size.height > screen_height)
		y = screen_height - menu_size.height;
	GtkAllocation menu_position;
	menu_position.x = x;
	menu_position.y = y;
	gtk_menu_popup (GTK_MENU (menu), NULL, NULL, (GtkMenuPositionFunc)
		tb_menu_position, &menu_position, button, time);
	return TRUE;
}

static gboolean tb_menu_popup_mnemonic_cb (GtkWidget *button, gboolean arg,
	E2_ToolbarRuntime *rt)
{
	gint x, y;
	e2_utils_get_abs_pos (button, &x, &y);
	y += button->allocation.height;
	menu_popup (button, x, y, rt, 0, 0);
	return FALSE;
}

static gboolean tb_menu_popup_cb (GtkWidget *button, GdkEventButton *event,
	E2_ToolbarRuntime *rt)
{
	if (event->button == 1)
	{
		gint x = event->x_root - event->x;
		gint y = event->y_root - event->y + button->requisition.height;
		return menu_popup (button, x, y, rt, event->button, event->time);
	}
	return FALSE;
}

static void resize_combo (GtkWidget *child, E2_ToolbarRuntime *rt)
{
	GtkWidget *command_line = g_object_get_data (G_OBJECT (child), "command_line");
	if (command_line != NULL)
	{
		GtkRequisition req_cl, req_hb;
		gtk_widget_size_request (GTK_WIDGET (rt->toolbar), &req_hb);
		if (req_hb.width == rt->alloc->width) return;
		gtk_widget_size_request (GTK_WIDGET (command_line), &req_cl);
		
		gint new_width = rt->alloc->width - (req_hb.width - req_cl.width);
		gint max_width = (gint) g_object_get_data (G_OBJECT (child), "cl_max_width");
		if ((max_width != -1) && (new_width > max_width)) new_width = max_width;
		gint min_width = (gint) g_object_get_data (G_OBJECT (child), "cl_min_width");
		if ((min_width != -1) && (new_width < min_width)) new_width = min_width;
		if (new_width < 1) new_width = 1;

		if (req_cl.width != new_width)
			gtk_widget_set_size_request (command_line, new_width, -1);
	}
}

static void hide_last_button (E2_ToolbarRuntime *rt)
{
	printd (DEBUG, "hide_last_button (rt:)");
	GList *children = gtk_container_get_children (GTK_CONTAINER (rt->toolbar));
	gint listlen = g_list_length (children);
	if (listlen - rt->buttons_num_hidden > 0)
	{
		GList *last = g_list_nth (children,
			listlen - rt->buttons_num_hidden - 1);
		GtkWidget *button = last->data;
		GtkRequisition req;
		gtk_widget_size_request (GTK_WIDGET (rt->toolbar), &req);
		gtk_widget_hide (button);
		GtkRequisition req2;
		gtk_widget_size_request (GTK_WIDGET (rt->toolbar), &req2);
		//FIXME: use this silly code to find out the real size the button
		//has inside the toolbar
		rt->button_last_width = req.width - req2.width;
		rt->button_last_height = req.height - req2.height;
		rt->buttons_num_hidden++;
		rt->buttons_hidden = g_list_append (rt->buttons_hidden, button);
	}
}

static void show_last_button (E2_ToolbarRuntime *rt)
{
	if (rt->buttons_hidden == NULL)
		return;
	GList *last = g_list_last (rt->buttons_hidden);
	gtk_widget_show (last->data);
	rt->buttons_num_hidden--;
	if (rt->buttons_num_hidden < 1)
	{
		rt->buttons_num_hidden = 0;
		g_list_free (rt->buttons_hidden);
		rt->buttons_hidden = NULL;
		rt->is_too_small = FALSE;
		gtk_widget_hide (rt->button_rest);
		printd (NOTICE, "everything visible again");
	} else
		g_list_delete_link (rt->buttons_hidden, last);
}

static void add_rest_button (E2_ToolbarRuntime *rt)
{
	printd (DEBUG, "add_rest_button (rt:)");
	if (rt->is_too_small)
		return;
	printd (NOTICE, "toolbar too small");
	if (rt->button_rest != NULL)
	{
		gtk_widget_show (rt->button_rest);
		rt->is_too_small = TRUE;
		return;
	}
	rt->button_rest = _e2_button_get (_e2_option_int_get (rt->style) == 0 ? "" : "_rest",
		_e2_option_bool_get (rt->hori) ? GTK_STOCK_GOTO_LAST : GTK_STOCK_GOTO_BOTTOM,
		_e2_option_int_get (rt->isize) + 1, "show rest buttons", NULL, NULL,
			(_e2_option_int_get (rt->style) == 2 ? E2_BUTTON_ICON_ABOVE_LABEL : 0) |
			(_e2_option_int_get (rt->style) == 0 ? E2_BUTTON_SHOW_MISSING_ICON : 0));
	gtk_button_set_relief (GTK_BUTTON (rt->button_rest),
		_e2_option_bool_get (rt->relief) ? GTK_RELIEF_NORMAL : GTK_RELIEF_NONE);
	g_signal_connect (rt->button_rest, "button-press-event",
		G_CALLBACK (tb_menu_popup_cb), rt);
	g_signal_connect (rt->button_rest, "mnemonic-activate",
		G_CALLBACK (tb_menu_popup_mnemonic_cb), rt);
	gtk_box_pack_end (GTK_BOX (rt->toolbar_container_box), rt->button_rest, FALSE, FALSE, 0);
	gtk_widget_show_all (rt->button_rest);
	rt->is_too_small = TRUE;
}

static void resize_cb (GtkWidget *toolbar, GtkAllocation *alloc, E2_ToolbarRuntime *rt)
{
	if (rt->has_command_line && !(GTK_HANDLE_BOX
		(rt->toolbar_container)->child_detached) && !rt->is_too_small &&
		!(_e2_option_int_get (rt->space) == 1))
	{
		if (alloc->width < 10)
			return;
		rt->alloc = alloc;
		gulong handler = g_signal_handler_find (rt->toolbar, G_SIGNAL_MATCH_FUNC, 
			0, 0, NULL, resize_cb, NULL);
		g_signal_handler_block (rt->toolbar, handler);
		gtk_container_foreach (GTK_CONTAINER (rt->toolbar), (void *) resize_combo, rt);
		g_signal_handler_unblock (rt->toolbar, handler);
	}
	if (_e2_option_int_get (rt->space) == 2)
	{
		GtkRequisition req;
		gtk_widget_size_request (toolbar, &req);
		if (_e2_option_bool_get (rt->hori))
		{
			gulong handler = g_signal_handler_find (rt->toolbar,
			G_SIGNAL_MATCH_FUNC, 0, 0, NULL, resize_cb, NULL);
			g_signal_handler_block (rt->toolbar, handler);
			if (alloc->width < req.width)
			{
				add_rest_button (rt);
				hide_last_button (rt);
			} else while ((alloc->width > (req.width + rt->button_last_width + 1))
				&& rt->is_too_small)
			{
				show_last_button (rt);
				alloc = &toolbar->allocation;
				req = toolbar->requisition;
			}
			g_signal_handler_unblock (rt->toolbar, handler);
		} else
		{
			gulong handler = g_signal_handler_find (rt->toolbar,
			G_SIGNAL_MATCH_FUNC, 0, 0, NULL, resize_cb, NULL);
			g_signal_handler_block (rt->toolbar, handler);
			if (alloc->height < req.height)
			{
				add_rest_button (rt);
				hide_last_button (rt);
			} else while ((alloc->height > (req.height + rt->button_last_height + 1))
				&& rt->is_too_small)
			{
				show_last_button (rt);
				alloc = &toolbar->allocation;
				req = toolbar->requisition;
			}
			g_signal_handler_unblock (rt->toolbar, handler);
		}
	}
}

  /*****************/
 /***** utils *****/
/*****************/

static void pack_and_prio (GtkWidget *box, E2_ToolbarRuntime *rt)
{
	gtk_box_pack_start (GTK_BOX (box), rt->toolbar_container, FALSE, FALSE, 0);
	gtk_box_reorder_child (GTK_BOX (box), rt->toolbar_container, _e2_option_int_get (rt->priority));
}

static void get_option_row (E2_ToolbarRuntime *rt, GtkTreeModel *model, GtkTreeIter *iter,
	gchar **name, gchar **icon, gchar **tip, gchar **type, gchar **arg)
{
	gchar *_name, *_icon, *_tip, *_type, *_arg;
	if (type == NULL)
		gtk_tree_model_get (model, iter, 0, &_name, 1, &_icon, 2, &_tip,
			3, &_arg, -1);
	else
	{
		gtk_tree_model_get (model, iter, 0, &_name, 1, &_icon, 2, &_tip,
			3, &_type, 4, &_arg, -1);
		*type = _type;
	}
	*name = e2_utils_replace_env_vars (_name);
	*icon = _icon;
	*tip = e2_utils_replace_env_vars (_tip);
	*arg = e2_utils_replace_env_vars (_arg);
	if ((*tip != NULL) && (**tip == '\0'))
	{
		g_free (*tip);
		*tip = NULL;
	}
	g_free (_name);
	g_free (_tip);
	g_free (_arg);
}

static GtkWidget *toolbar_add_button (E2_ToolbarRuntime *rt, gchar *name,
	gchar *icon, gchar *tip, gpointer data)
{
	GtkWidget *button;
	if (_e2_option_bool_get (rt->same))
	{
		GtkWidget *image = e2_widget_get_icon (icon,
			_e2_option_int_get (rt->isize) + 1);
		if ((_e2_option_int_get (rt->style) == 0) && (image == NULL))
			image = e2_widget_get_icon (GTK_STOCK_MISSING_IMAGE,
				_e2_option_int_get (rt->isize) + 1);
		button = gtk_toolbar_append_item (rt->toolbar, name, tip, NULL, image,
			G_CALLBACK (e2_action_run), data);
	} else
	{
		button = _e2_button_get (_e2_option_int_get (rt->style) == 0 ? "" : name,
			icon, _e2_option_int_get (rt->isize) + 1, NULL, e2_action_run, data,
			(_e2_option_int_get (rt->style) == 2 ? E2_BUTTON_ICON_ABOVE_LABEL : 0) |
			(_e2_option_int_get (rt->style) == 0 ? E2_BUTTON_SHOW_MISSING_ICON : 0));
		gtk_toolbar_append_widget (rt->toolbar, button, tip, NULL);
	}
	g_object_set_data_full (G_OBJECT (button), "free-callback-data", data,
		(GDestroyNotify) e2_action_free_runtime);
	gtk_button_set_relief (GTK_BUTTON (button), _e2_option_bool_get (rt->relief) ?
		GTK_RELIEF_NORMAL : GTK_RELIEF_NONE);
	return button;
}

  /******************/
 /***** public *****/
/******************/

void e2_toolbar_add_bookmarks (E2_OptionSet *set, E2_ToolbarRuntime *rt,
	GtkTreeIter *iter, E2_Action *action)
{
	do
	{
		gchar *name, *icon, *tip, *path;
		get_option_row (rt, set->ex.tree.model, iter, &name, &icon, &tip, NULL, &path);
		GtkWidget *button = toolbar_add_button (rt, name, icon, tip,
			e2_action_pack_runtime (action, path, g_free));
		g_signal_connect_data (button, "button-press-event", G_CALLBACK (e2_bookmark_click_cb),
			gtk_tree_model_get_path (set->ex.tree.model, iter), (GClosureNotify) gtk_tree_path_free, 0);
		g_object_set_data (G_OBJECT (button), "bookmark-path", path);
		if (gtk_tree_model_iter_has_child (set->ex.tree.model, iter))
		{
			GtkWidget *menu = gtk_menu_new ();
			GtkTreeIter iter2;
			if (gtk_tree_model_iter_children (set->ex.tree.model, &iter2, iter))
				e2_menu_add_bookmarks (action, set, menu, &iter2);
			g_object_set_data_full (G_OBJECT (button), "menu", menu,
					(GDestroyNotify) gtk_widget_destroy);
			g_signal_connect (button, "button-press-event", G_CALLBACK (tb_menu_popup_cb), rt);
			g_signal_connect (button, "mnemonic-activate", G_CALLBACK (tb_menu_popup_mnemonic_cb), rt);
		}
		g_free (name);
		g_free (icon);
		g_free (tip);
	} while ((gtk_tree_model_iter_next (set->ex.tree.model, iter)) == TRUE);
}


void e2_toolbar_add_menu_item (GtkWidget *menu, GtkTreeModel *model, GtkTreeIter *iter, gint level,
	E2_ToolbarRuntime *rt)
{
	do
	{
		gchar *name, *icon, *tip, *type, *arg;
		get_option_row (rt, model, iter, &name, &icon, &tip, &type, &arg);

		E2_Action *action = e2_action_get (type);
		if (action != NULL) 
		{
			switch (action->type)
			{
				case E2_ACTION_TYPE_SUBMENU:
				{
					GtkWidget *item = e2_menu_add (menu, name,
						icon, tip, NULL, NULL);
					GtkWidget *menu2 = gtk_menu_new ();
					GtkTreeIter iter2;
					gtk_tree_model_iter_children (model, &iter2, iter);
					e2_toolbar_add_menu_item (menu2, model, &iter2, level + 1, rt);
					gtk_menu_item_set_submenu (GTK_MENU_ITEM (item), menu2);
					break;
				}
				case E2_ACTION_TYPE_BOOKMARKS:
				{
					E2_OptionSet *set = e2_option_get ("bookmarks");
					GtkTreeIter iter;
								if ((arg != NULL) && (*arg != '\0'))
					{
						if (e2_tree_find_iter_from_str (set->ex.tree.model, 0, arg, &iter, TRUE))
						{
							GtkTreeIter iter2;
							if (gtk_tree_model_iter_children (set->ex.tree.model, &iter2, &iter))
							{
								e2_menu_add_bookmarks (action, set, menu, &iter2);
								rt->has_bookmarks = TRUE;
							}

						}
					} else if (gtk_tree_model_get_iter_first (set->ex.tree.model, &iter))
					{
							e2_menu_add_bookmarks (action, set, menu, &iter);
							rt->has_bookmarks = TRUE;
					}
				}
					break;
				case E2_ACTION_TYPE_PLUGINS:
					create_plugins_menu (menu);
					break; 
				case E2_ACTION_TYPE_SEPARATOR:
					e2_menu_add_separator (menu);
					break;
				case E2_ACTION_TYPE_TOGGLE:
				case E2_ACTION_TYPE_ITEM:
					{
						E2_ActionRuntime *actionrt = e2_action_pack_runtime (action, arg, NULL);
						GtkWidget *item = e2_menu_add (menu, name, icon, tip,
							G_CALLBACK (e2_action_run), actionrt);
						g_object_set_data_full (G_OBJECT (item), "free-callback-data", actionrt,
							(GDestroyNotify) e2_action_free_runtime);
					}
					break;
				default:
					break;
			}
		}
		//cleanup
		g_free (name);
		g_free (icon);
		g_free (tip);
		g_free (type);
	} while ((gtk_tree_model_iter_next (model, iter)) == TRUE);
}

void e2_toolbar_add_item (GtkTreeModel *model, GtkTreeIter *iter, gint level,
	E2_ToolbarRuntime *rt)
{
	do
	{
		gchar *name, *icon, *tip, *type, *arg;
		get_option_row (rt, model, iter, &name, &icon, &tip, &type, &arg);

		E2_Action *action = e2_action_get (type);
		if (action != NULL) 
		{
			GtkWidget *button = NULL;
			switch (action->type)
			{
				case E2_ACTION_TYPE_COMMAND_LINE:
				{
					E2_CommandLineRuntime *cl_rt = e2_command_line_create (name,
						action->data, action->func, action->has_arg ?
						E2_COMMAND_LINE_ORIGINAL : E2_COMMAND_LINE_DIR_LINE, action->data2);
					GtkWidget *command_line = GTK_WIDGET (cl_rt->cl);
					button = command_line;
					gtk_toolbar_append_widget (rt->toolbar, command_line, NULL, NULL);
					g_object_set_data_full (G_OBJECT (rt->toolbar), cl_rt->name,
						cl_rt, (GDestroyNotify) e2_command_line_destroy);
					//fallback values
					gint min = 100, max = 200;
					if (strlen (arg) > 0)
					{
						gchar **split = g_strsplit (arg, ",", 2);
						gchar *end = NULL;
						gint i;
						if (split[0])
						{
							i = (gint) g_ascii_strtoull (split[0], &end, 10);
							if (end != split[0])
								min = i;
						}
						if (split[1])
						{
							if ((g_str_equal (split[1], "expand")) || (g_str_equal (split[1], "*")))
								max = -1;
							else
							{
								i = (gint) strtoull (split[1], &end, 10);	
								if (end != split[0])
									max = i;
							}
						}
						g_strfreev (split);
					}
					gtk_widget_set_size_request (command_line, max, -1);
					g_object_set_data (G_OBJECT (command_line), "command_line", command_line);
					g_object_set_data (G_OBJECT (command_line), "cl_min_width", (void *) min);
					g_object_set_data (G_OBJECT (command_line), "cl_max_width", (void *) max);
					rt->has_command_line = TRUE;
				}
					break;
				case E2_ACTION_TYPE_SUBMENU:
				{
					GtkWidget *menu = gtk_menu_new ();
					GtkTreeIter iter2;
					if (gtk_tree_model_iter_children (model, &iter2, iter))
					{
						e2_toolbar_add_menu_item (menu, model, &iter2, level + 1, rt);
					}
					button = toolbar_add_button (rt, name, icon, tip, NULL);
					g_object_set_data_full (G_OBJECT (button), "menu", menu,
							(GDestroyNotify) gtk_widget_destroy);
					g_signal_connect (button, "button-press-event", G_CALLBACK (tb_menu_popup_cb), rt);
					g_signal_connect (button, "mnemonic-activate", G_CALLBACK (tb_menu_popup_mnemonic_cb), rt);
					break;
				}
				case E2_ACTION_TYPE_SEPARATOR:
				{
					GtkWidget *sep, *align;
					if (_e2_option_bool_get (rt->hori))
					{
						sep = gtk_vseparator_new ();
						align = gtk_alignment_new (0.5, 0.5, 1.0, 0.6);
					} else
					{
						sep = gtk_hseparator_new ();
						align = gtk_alignment_new (0.5, 0.5, 0.6, 1.0);
					}
					gtk_container_add (GTK_CONTAINER (align), sep);
					gtk_container_set_border_width (GTK_CONTAINER (align), 2);
					gtk_toolbar_append_widget (rt->toolbar, align, NULL, NULL);
					break;
				}
				case E2_ACTION_TYPE_BOOKMARKS:
					{
						E2_OptionSet *set = e2_option_get ("bookmarks");
						GtkTreeIter iter;
						if ((arg != NULL) && (*arg != '\0'))
						{
							if (e2_tree_find_iter_from_str (set->ex.tree.model, 0, arg, &iter, TRUE))
							{
								GtkTreeIter iter2;
								if (gtk_tree_model_iter_children (set->ex.tree.model, &iter2, &iter))
								{
									rt->has_bookmarks = TRUE;
									e2_toolbar_add_bookmarks (set, rt, &iter2, action);
								}
							}
						} else if (gtk_tree_model_get_iter_first (set->ex.tree.model, &iter))
						{
							rt->has_bookmarks = TRUE;
							e2_toolbar_add_bookmarks (set, rt, &iter, action);
						}
					}
					break;
				case E2_ACTION_TYPE_TOGGLE:
				case E2_ACTION_TYPE_ITEM:
					button = toolbar_add_button (rt, name, icon, tip,
						e2_action_pack_runtime (action, arg, g_free));
					break;
				default:
					break;
			}
			if (button != NULL)
				g_object_set_data_full (G_OBJECT (button), "path",
					gtk_tree_model_get_path (model, iter),
					(GDestroyNotify) gtk_tree_path_free);
			//cleanup
			if ((action->type != E2_ACTION_TYPE_ITEM) &&
				(action->type != E2_ACTION_TYPE_TOGGLE))
				g_free (arg);
		} 
		//cleanup
		g_free (name);
		g_free (type);
		g_free (icon);
		g_free (tip);
	} while ((gtk_tree_model_iter_next (model, iter)) == TRUE);
}

void e2_toolbar_create (E2_ToolbarRuntime *rt, gchar *option,
	gchar *group_parent, gint data[])
{
	rt->toolbar = NULL;
	rt->toolbar_container = NULL;
	rt->has_bookmarks = FALSE;
	rt->has_command_line = FALSE;
	rt->is_too_small = FALSE;
	rt->button_rest = NULL;
	rt->buttons_hidden = NULL;
	rt->buttons_num_hidden = 0;

	//main toolbar tree option registered locally
	//(config/e2_options_default.c)
	rt->option = option;
	rt->group = group_parent;
	rt->set = e2_option_get (option);

	gchar *group = g_strconcat (group_parent, ".misc:options", NULL);
	gchar *name, *desc, *tip;
	
	//show toolbar
	name = g_strconcat (option, "-show", NULL);
	desc = g_strconcat ("show ", option, NULL);
	tip = g_strconcat ("if you deactivate this, the ", option, " will be hidden", NULL);
	rt->show = e2_option_bool_register (name, g_strdup (group), desc, tip, NULL, data[0]);
	rt->show->should_be_freed = TRUE;
	//show tooltips
	name = g_strconcat (option, "-tooltips", NULL);
	desc = g_strconcat ("show tooltips for ", option, " items", NULL);
	tip = g_strconcat ("if deactivated, this will generally turn off all tooltips for the ", option, NULL);
	rt->tooltips = e2_option_bool_register (name, g_strdup (group), desc, tip, NULL, data[1]);
	rt->tooltips->should_be_freed = TRUE;
	//space
	name = g_strconcat (option, "-space", NULL);
	desc = g_strdup ("space handling");
	tip = g_strconcat ("change type of the ", option, " space handling", NULL);
	gchar *opt_toolbar_space[] =
		{"none", "use scrollbars", "use rest button", NULL};
	rt->space = e2_option_sel_register (name, group, desc, tip,
		NULL, data[2], opt_toolbar_space);
	rt->space->should_be_freed = TRUE;

	group = g_strconcat (group_parent, ".misc:position", NULL);
	
	//type
	name = g_strconcat (option, "-type", NULL);
	desc = g_strconcat (option, " type", NULL);
	tip = g_strconcat ("change type of the ", option, NULL);
	gchar *opt_toolbar_type[] =
		{"main toolbar", "big panebar", "small panebar 1", "small panebar 2", NULL};
	rt->type = e2_option_sel_register (name, g_strdup (group), desc, tip,
		NULL, data[3], opt_toolbar_type);
	rt->type->should_be_freed = TRUE;
	//toolbar horizontal
	name = g_strconcat (option, "-hori", NULL);
	desc = g_strconcat (option, " horizontal", NULL);
	tip = g_strconcat ("align ", option, " elements horizontally", NULL);
	rt->hori = e2_option_bool_register (name, g_strdup (group), desc, tip, NULL, data[4]);
	rt->hori->should_be_freed = TRUE;
	//priority
	name = g_strconcat (option, "-priority", NULL);
	desc = g_strconcat (option, " priority", NULL);
	tip = g_strdup ("the priority changes the order of toolbars");
	rt->priority = e2_option_int_register (name, group, desc, tip, NULL, data[5], 0, 10);
	rt->priority->should_be_freed = TRUE;
	
	group = g_strconcat (group_parent, ".misc:style", NULL);
	
	//buttons relief
	name = g_strconcat (option, "-relief", NULL);
	desc = g_strconcat (option, " buttons have a relief", NULL);
	tip = g_strconcat ("usually ", option, " buttons do not have a relief, but you may like it", NULL);
	rt->relief = e2_option_bool_register (name, g_strdup (group), desc, tip, NULL, data[6]);
	rt->relief->should_be_freed = TRUE;
	//buttons same size
	name = g_strconcat (option, "-same", NULL);
	desc = g_strconcat (option, " buttons have the same size", NULL);
	tip = g_strconcat ("for a horizontal ", option, " buttons with equal width look bad", NULL);
	rt->same = e2_option_bool_register (name, g_strdup (group), desc, tip, NULL, data[7]);
	rt->same->should_be_freed = TRUE;
	//buttons style
	name = g_strconcat (option, "-style", NULL);
	desc = g_strdup ("button style");
	tip = g_strdup ("the default is 'icons above labels'");
	gchar *opt_toolbar_style[] =
		{"icons only", "text only", "icons above text", "icons aside text", NULL};
	rt->style = e2_option_sel_register (name, g_strdup (group), desc, tip,
		NULL, data[8], opt_toolbar_style);
	rt->style->should_be_freed = TRUE;
	//buttons icon size
	name = g_strconcat (option, "-isize", NULL);
	desc = g_strdup ("icon size");
	tip = g_strdup ("the default is 'toolbar small'");
	gchar *opt_toolbar_isize[] =
		{"menu", "toolbar small", "toolbar large", "button", "dnd", "dialog", NULL};
	rt->isize = e2_option_sel_register (name, group, desc, tip,
		NULL, data[9], opt_toolbar_isize);
	rt->isize->should_be_freed = TRUE;

	name = g_strconcat ("toggle ", option, NULL);
	_e2_action_register (name, E2_ACTION_TYPE_TOGGLE,
		toggle_toolbar_cb, rt, FALSE, 0, rt->show->name);
	g_free (name);

}

void _e2_toolbar_create (E2_ToolbarRuntime *rt)
{
	if (!_e2_option_bool_get (rt->show)) return;
	
	rt->blocked = FALSE;
	rt->hidden = FALSE;
	rt->is_too_small = FALSE;
	rt->button_rest = NULL;
	if (rt->buttons_hidden != NULL)
	{
		g_list_free (rt->buttons_hidden);
		rt->buttons_hidden = NULL;
	}
	rt->buttons_num_hidden = 0;
	
	rt->toolbar = GTK_TOOLBAR (gtk_toolbar_new ());
	//setup toolbar
	gtk_toolbar_set_tooltips (rt->toolbar, _e2_option_bool_get (rt->tooltips));
	gtk_toolbar_set_style (rt->toolbar, (GtkToolbarStyle) _e2_option_int_get (rt->style));
	gtk_toolbar_set_icon_size (rt->toolbar, _e2_option_int_get (rt->isize) + 1);

	//handlebox as toolbar container
	rt->toolbar_container = gtk_handle_box_new ();
	gtk_handle_box_set_shadow_type (GTK_HANDLE_BOX (rt->toolbar_container), GTK_SHADOW_NONE);
	g_signal_connect (G_OBJECT (rt->toolbar), "size-allocate", G_CALLBACK (resize_cb), rt);

	//setup toolbar orientation
	if (_e2_option_bool_get (rt->hori))
	{
		rt->toolbar_container_box = gtk_hbox_new (FALSE, 0);
		gtk_toolbar_set_orientation (rt->toolbar, GTK_ORIENTATION_HORIZONTAL);
		gtk_handle_box_set_handle_position (GTK_HANDLE_BOX (rt->toolbar_container), GTK_POS_LEFT);
		gtk_handle_box_set_snap_edge (GTK_HANDLE_BOX (rt->toolbar_container), GTK_POS_TOP);
	} else
	{
		rt->toolbar_container_box = gtk_vbox_new (FALSE, 0);
		gtk_toolbar_set_orientation (rt->toolbar, GTK_ORIENTATION_VERTICAL);
		gtk_handle_box_set_handle_position (GTK_HANDLE_BOX (rt->toolbar_container), GTK_POS_TOP);
		gtk_handle_box_set_snap_edge (GTK_HANDLE_BOX (rt->toolbar_container), GTK_POS_LEFT);
	}
	gtk_container_add (GTK_CONTAINER (rt->toolbar_container),
		rt->toolbar_container_box);

	//scrollbars
	if (_e2_option_int_get (rt->space) == 1)
	{
		GtkWidget *sw;
		if (_e2_option_bool_get (rt->hori))
			sw = e2_widget_get_sw (GTK_POLICY_AUTOMATIC, GTK_POLICY_NEVER);
		else
			sw = e2_widget_get_sw (GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
		gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (sw), GTK_SHADOW_NONE);
		gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW (sw), GTK_WIDGET (rt->toolbar));
		gtk_box_pack_start (GTK_BOX (rt->toolbar_container_box), sw, TRUE, TRUE, 0);
	} else
		gtk_box_pack_start (GTK_BOX (rt->toolbar_container_box), GTK_WIDGET (rt->toolbar), TRUE, TRUE, 0);

	//add toolbar to the main window
	gint type = _e2_option_int_get (rt->type);
	switch (type)
	{
		case 3:
			if (_e2_option_bool_get (rt->hori))
				pack_and_prio (app.pane2.inner_box, rt);
			else
				pack_and_prio (app.pane2.outer_box, rt);
			break;
		case 2:
			if (_e2_option_bool_get (rt->hori))
				pack_and_prio (app.pane1.inner_box, rt);
			else
				pack_and_prio (app.pane1.outer_box, rt);
			break;
		case 1:
			if (_e2_option_bool_get (rt->hori))
			{
				if (e2_option_bool_get ("panes-horizontal"))
					pack_and_prio (app.window.panes_inner_box, rt);
				else
					pack_and_prio (app.window.panes_outer_box, rt);
			}
			else
			{
				if (e2_option_bool_get ("panes-horizontal"))
					pack_and_prio (app.window.panes_outer_box, rt);
				else
					pack_and_prio (app.window.panes_inner_box, rt);
			}
			break;
		default:
			if (_e2_option_bool_get (rt->hori))
				pack_and_prio (app.vbox_main, rt);
			else
				pack_and_prio (app.hbox_main, rt);
	}

	//add items
	GtkTreeIter iter;
	if ((gtk_tree_model_get_iter_first (rt->set->ex.tree.model, &iter)) == TRUE)
		e2_toolbar_add_item (rt->set->ex.tree.model, &iter, 0, rt);
	
	//context menu
	g_signal_connect (G_OBJECT (rt->toolbar_container), "button-press-event", G_CALLBACK (click_cb), rt);
	
	GList *children = rt->toolbar->children;
	while (children != NULL)
	{
		GtkToolbarChild *child = children->data;
		if ((GTK_IS_BUTTON (child->widget)) && (GTK_IS_LABEL (child->label)))
		{
			gtk_label_set_use_markup (GTK_LABEL (child->label), TRUE);
			gtk_label_set_use_underline (GTK_LABEL (child->label), TRUE);
		}
		children = g_list_next (children);
	}

	gtk_widget_show_all (rt->toolbar_container);

	//FIXME: add minimum sizes to panes if the toolbar is a small panebar
/*	if (type > 1)
	{
		
	} */
}

void _e2_toolbar_recreate (E2_ToolbarRuntime *rt)
{
	e2_toolbar_destroy (rt);
	_e2_toolbar_create (rt);
}

void e2_toolbar_destroy (E2_ToolbarRuntime *rt)
{
	if (rt->toolbar != NULL)
	{
		gtk_widget_destroy (GTK_WIDGET (rt->toolbar));
		rt->toolbar = NULL;
	}
	if (rt->toolbar_container != NULL)
	{
		gtk_widget_destroy (rt->toolbar_container);
		rt->toolbar_container = NULL;
	}
}
