/* $Id: emelfm2.h,v 1.35 2004/08/09 23:43:19 cairo Exp $
 *
 * Copyright (C) 2004 tooar <tooar@gmx.net>
 * Copyright (C) 1999 Michael Clark.
 *
 * This file is part of emelFM2.
 * emelFM2 is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * emelFM2 is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 * 9-22-2000: Modified by Aurelien Gateau
 *						Added code for show hidden toggle button
 */

#ifndef __EMELFM2_H__
#define __EMELFM2_H__

#define _GNU_SOURCE
#include <gtk/gtk.h>
#include <gdk/gdkkeysyms.h>
#include <gmodule.h>

#define GETTEXT_PACKAGE "emelfm2"
#include <glib/gi18n-lib.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <limits.h>

#include "build.h"
#include "utils/debug.h"
#include "utils/e2_button.h"
#include "utils/e2_combobox.h"
#include "utils/e2_hook.h"
#include "utils/e2_list.h"
#include "utils/e2_menu.h"
#include "utils/e2_tree.h"
#include "utils/e2_utf8.h"
#include "utils/e2_utils.h"
#include "utils/e2_widget.h"

#include "config/e2_cl_option.h"

#include "command/e2_command.h"

#include "e2_toolbar.h"
#include "e2_pane.h"
#include "e2_output.h"
#include "e2_window.h"
#include "e2_alias.h"
#include "e2_command_line.h"
#include "e2_cache.h"
#include "e2_bookmark.h"
#include "e2_keybinding.h"

#include "dialogs/e2_dialog.h"

//FIXME:some new constants
#define E2_PADDING_LARGE 8
#define E2_PADDING 5
#define E2_PADDING_SMALL 3
#define E2_PADDING_XSMALL 2

//FIXME: old constants, remove
#define NAME_MAX 255
#define MAX_LEN 1024
#define MAX_COLUMNS 8

//some nice helper/utility macros

//#define _STR2UTF(d) g_filename_to_utf8 (d, -1, NULL, NULL, NULL)
//#define _UTF2STR(d) g_filename_from_utf8 (d, -1, NULL, NULL, NULL)
//#define STR2UTF(d) g_locale_to_utf8 (d, -1, NULL, NULL, NULL)
//#define UTF2STR(d) g_locale_from_utf8 (d, -1, NULL, NULL, NULL)

#define MESC(d) g_markup_escape_text (d, -1)
#define WAIT_FOR_EVENTS while (gtk_events_pending ()) { gtk_main_iteration (); }

#define RUN_ONCE_CHECK(ret) \
	static gboolean run_once_check_init = FALSE; \
	if (run_once_check_init) \
		return ret; \
	else \
		run_once_check_init = TRUE;

typedef enum { FILENAME = 0, SIZE, PERM, OWNER, GROUP,
	MODIFIED, ACCESSED, CHANGED } ColumnFlags;

typedef enum { GT, LT, EQ } Operator;

typedef enum
{
	YES = 1 << 0,
	OK = 1 << 1,
	YES_TO_ALL = 1 << 2,
	APPLY_TO_ALL = 1 << 3,
	NO = 1 << 4,
	CANCEL = 1 << 5
} DialogButtons;

typedef struct _FileInfo
{
	gchar filename[NAME_MAX];
	struct stat statbuf;
} FileInfo;

typedef struct _FileType
{
	gchar description[NAME_MAX];
	gchar extensions[NAME_MAX];
	gchar actions[NAME_MAX];
} FileType;

typedef struct _Column
{
	gchar *title;
	gint size;
	gint id;
	gint is_visible;
	GtkCListCompareFunc sort_func;
} Column;

typedef struct _Plugin
{
	gchar filename[PATH_MAX+NAME_MAX];
	gchar *name;
	gchar *description;
	gboolean show_in_menu;
	gboolean load;
	GModule *module;
	void (*plugin_cb)();
} Plugin;

typedef struct _FileView
{
	GtkWidget *clist;
	GtkWidget *sort_arrows[MAX_COLUMNS];
	GtkWidget *filter_menu_item;
	GtkWidget *hidden_button;
	GList *old_selection;
	GList *tagged;
	GList *dir_history;
	gchar last_find[NAME_MAX];
	gchar dir[PATH_MAX];
	time_t dir_mtime;

	gboolean filter_directories;
	struct
	{
		gchar pattern[NAME_MAX];
		gboolean invert_mask;
		gboolean case_sensitive;
		gboolean active;
	} filename_filter;
	struct
	{
		off_t size;
		Operator op;
		gboolean active;
	} size_filter;
	struct
	{
		time_t time;
		Operator op;
		enum {MTIME, ATIME, CTIME} time_type;
		gboolean active;
	} date_filter;

	gint row;
	gint last_row;
	gint selected_rows;
	gboolean show_hidden;
} FileView;

typedef struct _emelFM2
{
	FileView left_view;
	FileView right_view;
	GtkWidget *main_window;
	//pair of main boxes that can be surrounded by toolbars
	GtkWidget *hbox_main;
	GtkWidget *vbox_main;
	
	E2_WindowRuntime window;
	E2_PaneRuntime pane1;
	E2_PaneRuntime pane2;
	E2_PaneRuntime *pane_current;
	E2_PaneRuntime *pane_other;
	GHookList hook_pane_focus_changed;
	E2_ToolbarRuntime toolbar;
	E2_ToolbarRuntime commandbar;
	E2_OutputRuntime output;
	E2_AliasRuntime aliases;
	//pointer to context menu when it is poped up
	GtkWidget *context_menu;
	GtkWidget *status_bar;
	GtkWidget *drag_op_menu;
	GtkTooltips *tooltips;
} emelFM2;

typedef struct _Config
{
	gchar version[20];
	time_t config_mtime;
	gint window_width;
	gint window_height;
	guint auto_refresh_id;
	guint check_config_id;
	GList *plugins;
	GList *filetypes;
} Config;


emelFM2 app;
Config cfg;
FileView *curr_view;
FileView *other_view;

Column all_columns[MAX_COLUMNS];

/* Colors */
GdkColor CLIST_COLOR;

/* Filetypes */
gchar *get_actions_for_ext(gchar *ext);
gchar *get_default_action_for_ext(gchar *ext);
FileType *get_filetype_for_ext(gchar *ext);
void add_filetype(gchar *ext, gchar *prog, gchar *desc);
void exec_filetype_action(gchar *action);

/* Plugins */
void destroy_plugin(Plugin *p);
void unload_all_plugins();
Plugin *load_plugin(gchar *filename);
void do_plugin_action(GtkWidget *widget, Plugin *p);
Plugin *get_plugin_by_name(gchar *name);

// Callbacks
void copy_cb(GtkWidget *widget);
void copy_as_cb(GtkWidget *widget);
void move_cb(GtkWidget *widget);
void move_as_cb(GtkWidget *widget);
void rename_cb(GtkWidget *widget);
void delete_cb(GtkWidget *widget);
void symlink_cb(GtkWidget *widget);
void symlink_as_cb(GtkWidget *widget);
void permissions_cb(GtkWidget *widget);
void ownership_cb(GtkWidget *widget);
void file_info_cb(GtkWidget *widget);
void sync_dirs_cb(GtkWidget *widget);
void view_cb(GtkWidget *widget);
void toggle_tag_cb();
void refresh_cb(GtkWidget *widget);
void command_cb(GtkWidget *widget);
void open_cb(GtkWidget *widget);
void open_with_cb(GtkWidget *widget);
void open_in_other_pane_cb(GtkWidget *widget);
//extern void mount_cb(GtkWidget *widget);
//extern void umount_cb(GtkWidget *widget);
gboolean configure_cb(GtkWidget *widget, gchar *page);
void quit_cb(GtkWidget *widget);
void find_cb(GtkWidget *widget);
//extern void execute_cb(GtkWidget *widget);
//extern void execute_with_args_cb(GtkWidget *widget);

/* FileView */
gboolean toggle_show_hidden(FileView *view);
gboolean toggle_show_hidden_cb(GtkWidget *widget, FileView *view);
void fileview_select_all ();
void fileview_select_all2 (GtkWidget *widget, FileView *view);
GtkWidget *create_file_view(FileView *view);
void switch_views ();

/* Filelist */
void get_perm_string(gchar *string, gint len, mode_t mode);
gint is_dir(FileInfo *info);
void change_dir(FileView *view, gchar *path, gchar *old);
gint name_sort(GtkCList *clist, gconstpointer row1, gconstpointer row2);
gint size_sort(GtkCList *clist, gconstpointer row1, gconstpointer row2);
gint date_sort(GtkCList *clist, gconstpointer row1, gconstpointer row2);
gint perm_sort(GtkCList *clist, gconstpointer row1, gconstpointer row2);
gint user_sort(GtkCList *clist, gconstpointer row1, gconstpointer row2);
gint group_sort(GtkCList *clist, gconstpointer row1, gconstpointer row2);
void sort_list(FileView *view, GtkCListCompareFunc,
	GtkSortType direction, gint col);
void focus_row(FileView *view, gint row, gboolean clear_selection,
	gboolean center, gboolean grab_focus);
void select_row_by_filename(FileView *view, gchar *filename);
GList *get_selection(FileView *view);
FileInfo *get_first_selected(FileView *view);
void initialize_filters(FileView *view);
void set_filter_menu_active(FileView *view);
void remove_filters(FileView *view);
void refresh_list(FileView *view);
void disable_refresh();
void reenable_refresh();
gint auto_refresh(gpointer data);
void load_dir_list(FileView *view);

/* File Ops */
gint file_copy(gchar *src, gchar *dest);
gint file_move(gchar *src, gchar *dest);
gint file_delete(gchar *file);
gint file_chmod(gchar *path, gchar *mode, gboolean recurse_dirs);
gint file_chown(gchar *path, uid_t owner_id, gid_t group_id,
											 gboolean recurse_dirs);
gint file_mkdir(gchar *path);
gint file_symlink(gchar *src, gchar *dest);
//extern gint file_mount(gchar *mount_point);
//extern gint file_umount(gchar *mount_point);

/* Dialogs */
GtkWidget *create_confirm_dialog (gchar *label_text, gint buttons);
gint op_with_ow_check(gchar *src, gchar *dest,
	 gint (*op_func)(gchar *, gchar *));
void create_permissions_dialog(FileInfo *info, guint *answer_ret,
	gboolean *recurse_ret, GString **mode_ret);
void create_ownership_dialog(FileInfo *info, guint *answer_ret, 
	uid_t *owner_ret, gid_t *group_ret, gboolean *recurse_ret);
void create_filetype_dialog(FileType **ft, gboolean write_file);
void create_filetype_action_dialog(gchar *name, gchar *action,
	gchar **arg1, gchar **arg2);
void create_init_filetype_dialog (FileInfo *info);
void create_add_ext_dialog();
void create_file_info_dialog();
void create_filename_filter_dialog(FileView *view);
void create_size_filter_dialog(FileView *view);
void create_date_filter_dialog(FileView *view);
void create_glob_dialog();
void create_rename_ext_dialog();
void create_user_prompt(gchar *prompt, gchar *init_text,
	 gboolean select_text, gchar **string);
GtkWidget * create_user_prompt2 (gchar *prompt, gchar *init_text,
	gboolean select_text, GtkWidget **entry);
void create_plugin_info_dialog(Plugin **p);
void create_open_query_dialog(gchar *filename);

/* Config Functions */
gint check_config_files();
void touch_config_dir();
void write_filetypes_file();
void write_plugins_file();
void write_config_file();
gboolean read_filetypes_file();
gboolean read_plugins_file();
gboolean read_config_file();

#endif /* __EMELFM2_H__ */
