/* $Id: debug.c,v 1.9 2004/03/25 21:23:50 cairo Exp $
 * 
 * Copyright (C) 2003-2004 tooar <tooar@gmx.net>
 * 
 * This file is part of emelFM2.
 * emelFM2 is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * emelFM2 is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with emelFM2; see the file COPYING. If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <time.h>
#include <sys/time.h>
#include "debug.h"
#include "e2_cl_option.h"

void printd_raw (int level, char *file, int line, const char *format, ...)
{
	if (level > e2_cl_options.debug_level) return;
	char *lvl;
	va_list args;
	if (e2_cl_options.verbose)
	{
		static long sec = 0;
		static long usec = 0;
		struct timeval t;
		
		gettimeofday (&t, NULL);
		if (!sec) sec = t.tv_sec;
		if (!usec) usec = t.tv_usec;
	
		long dsec = t.tv_sec - sec;
		long dusec = t.tv_usec - usec;

		sec = t.tv_sec;
		usec = t.tv_usec;

		if ((dsec > 0) && (dsec < 10))
			dusec += dsec * 1000000;

		if ((dusec < 10000) && (dsec == 0))
			printf ("%6ld ys %s/%d ", dusec, file, line);   
		else if ((dusec < (1000000)) && (dsec == 0))
			printf ("%6ld ms %s/%d ", (dusec / 1000), file, line);
		else
			printf ("%6ld s %s/%d ", (dusec / 1000000) + dsec, file, line);
	} 
	switch (level)
	{
		case ERROR:  lvl = "ERROR "; break;
		case WARN:   lvl = "WARN  "; break;
		case INFO:   lvl = "INFO  "; break;
		case NOTICE: lvl = "NOTICE"; break;
		default:     lvl = "DEBUG "; break;
	}
	printf ("[%s] ", lvl);
			
	va_start (args, format);	
	vprintf (format, args);
	va_end (args);	
	printf ("\n");
}
