/*
 * $Id: e2_button.c,v 1.8 2004/05/23 02:10:19 cairo Exp $
 * 
 * Copyright (C) 2004 tooar <tooar@gmx.net>
 * 
 * This file is part of emelFM2.
 * emelFM2 is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * emelFM2 is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with emelFM2; see the file COPYING. If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <stdlib.h>
#include <stdio.h>
#include "emelfm2.h"
#include "e2_button.h"

GtkWidget *e2_button_get (gchar *label, gchar *stock, gchar *tip,
	void *callback, void *data)
{
	GtkWidget *button = _e2_button_get (label, stock, GTK_ICON_SIZE_BUTTON,
		tip, callback, data, E2_BUTTON_CAN_DEFAULT | E2_BUTTON_CAN_FOCUS);
	gtk_widget_show_all (button);
	return button;
}

GtkWidget *_e2_button_get (gchar *label, gchar *stock, GtkIconSize size,
	gchar *tip, void *callback, void *data, E2_ButtonFlags flags)
{
	GtkWidget *button = gtk_button_new ();
	if (flags & E2_BUTTON_CAN_DEFAULT)
		GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
	else
		GTK_WIDGET_UNSET_FLAGS (button, GTK_CAN_DEFAULT);
	if (flags & E2_BUTTON_CAN_FOCUS)
		GTK_WIDGET_SET_FLAGS (button, GTK_CAN_FOCUS);
	else
		GTK_WIDGET_UNSET_FLAGS (button, GTK_CAN_FOCUS);

	if (callback != NULL)
		g_signal_connect_data (GTK_OBJECT (button), "clicked", G_CALLBACK (callback), data,
			(flags & E2_BUTTON_FREE_DATA) ? (GClosureNotify)  g_free : NULL, 0);
	
	GtkWidget *align = gtk_alignment_new (0.5, 0.5, 0.0, 0.0);
	GtkWidget *bbox;
	if (flags & E2_BUTTON_ICON_ABOVE_LABEL)
		bbox = gtk_vbox_new (FALSE, 2);
	else
		bbox = gtk_hbox_new (FALSE, 2);
	gtk_container_add (GTK_CONTAINER (align), bbox);

	gboolean need_align = FALSE;
	
	GtkWidget *img = e2_widget_get_icon (stock, size);
	if ((flags & E2_BUTTON_SHOW_MISSING_ICON) && (img == NULL))
		img = e2_widget_get_icon (GTK_STOCK_MISSING_IMAGE, size);
	if (img != NULL)
	{
		gtk_box_pack_start (GTK_BOX (bbox),
			img, FALSE, FALSE, 0);
		need_align = TRUE;
	}

	if ((label != NULL) && (*label != '\0'))
	{
		GtkWidget *lab = gtk_label_new ("");
		gtk_label_set_markup_with_mnemonic (GTK_LABEL (lab), label);
		gtk_box_pack_end (GTK_BOX (bbox), lab, TRUE, TRUE, 0);
		gtk_misc_set_alignment (GTK_MISC (lab), 0.5, 0.5);
		need_align = TRUE;
	}

	if (need_align)
		gtk_container_add (GTK_CONTAINER (button), align);

	e2_widget_set_tooltip (NULL, button, tip);

	return button;
}

GtkWidget *e2_button_add (GtkWidget *box, gint fill, gint pad, gchar *label,
	gchar *stock, gchar *tip, void *callback, void *data)
{
	GtkWidget *button = e2_button_get (label, stock, tip, callback, data);
	gtk_box_pack_start (GTK_BOX (box), button, fill, TRUE, pad);
	gtk_widget_show_all (button);
	return button;
}

GtkWidget *e2_button_get_toggle (gchar *label, gint state,
	void *func, gpointer data, gboolean check)
{
	GtkWidget *button;
	if (check)
		button = gtk_check_button_new_with_mnemonic (label);
	else
		button = gtk_toggle_button_new_with_mnemonic (label);
	GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
	GTK_WIDGET_SET_FLAGS (button, GTK_CAN_FOCUS);
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), state);
	if (func != NULL)
		g_signal_connect (GTK_OBJECT (button), "toggled",
			G_CALLBACK (func), data);
	return button;
}

GtkWidget *e2_button_add_toggle (GtkWidget *box, gchar *label, gint state,
	gint fill, gint pad, void *func, gpointer data, gboolean check)
{
	GtkWidget *button = e2_button_get_toggle (label, state, func, data, check);
	gtk_box_pack_start (GTK_BOX (box), button, fill, TRUE, pad);
	gtk_widget_show (button);
	return button;
}

GtkWidget *e2_button_add_radio (GtkWidget *box, gchar *label, GSList *group,
	gint fill, gint pad, void *func, gpointer data)
{
	GtkWidget *radio_button = gtk_radio_button_new_with_mnemonic (group, label);
	if (func != NULL)
		g_signal_connect (GTK_OBJECT (radio_button), "toggled",
			G_CALLBACK (func), data);
	gtk_box_pack_start (GTK_BOX (box), radio_button, fill, TRUE, pad);
	gtk_widget_show (radio_button);

	return radio_button;
}

