/* $Id: e2_menu.c,v 1.10 2004/07/13 10:38:35 cairo Exp $
 * 
 * Copyright (C) 2004 tooar <tooar@gmx.net>
 * 
 * This file is part of emelFM2.
 * emelFM2 is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * emelFM2 is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with emelFM2; see the file COPYING. If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/**
 * @file e2_menu.c 
 * @brief Menu utility functions
 *
 * This file contains utility and helper functions for menus.
 **/

#include <stdlib.h>
#include <stdio.h>
#include "emelfm2.h"
#include "e2_menu.h"

  /*********************/
 /***** callbacks *****/
/*********************/

/**
 * @brief callback to destroy a menu widget
 *
 * The menu widget @a menu is destroyed. This function might be used
 * as a callback for the @a "hide" signal of a menu.
 *
 * @param menu  the menu widget to destroy
 * @param data  this argument is ignored
 *
 * @return 
 **/
void e2_menu_destroy_cb (GtkWidget *menu, gpointer data)
{
	gtk_widget_destroy (menu);
}

void e2_menu_control_cb (GtkWidget *menu_item, gpointer data)
{
	e2_option_connect (data, gtk_check_menu_item_get_active
		(GTK_CHECK_MENU_ITEM (menu_item)));
}

  /******************/
 /***** public *****/
/******************/

GtkWidget *e2_menu_add (GtkWidget *menu, gchar *label, gchar *icon,
	gchar *tip, void *func, gpointer data)
{
	GtkWidget *menu_item;
	if ((icon != NULL) && (e2_option_bool_get ("menu-show-icons")))
	{
		menu_item = gtk_image_menu_item_new_with_mnemonic 
			(label != NULL ? label : "");
		GtkWidget *image = e2_widget_get_icon (icon,
			e2_option_int_get ("menu-isize") + 1);
		if (image != NULL)
			gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (menu_item), image);
	}
	else
		menu_item = gtk_menu_item_new_with_mnemonic (label);

	//turn on markup usage for menu items
	GList *children = gtk_container_get_children (GTK_CONTAINER (menu_item));
	GtkWidget *lab = children->data;
	gtk_label_set_use_markup (GTK_LABEL (lab), TRUE);
	
	if (func != NULL)
		g_signal_connect (GTK_OBJECT (menu_item), "activate", 
			G_CALLBACK (func), data);
	gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);
	gtk_widget_show_all (menu_item);

	e2_widget_set_tooltip (NULL, menu_item, tip);
	return menu_item;
}

GtkWidget *e2_menu_add_check (GtkWidget *menu,  gchar *label,
	gboolean state, void *func, gpointer data)
{
	GtkWidget *check = gtk_check_menu_item_new_with_mnemonic (label);
	if (menu != NULL)
		gtk_menu_shell_append (GTK_MENU_SHELL (menu), check);
	gtk_check_menu_item_set_state (GTK_CHECK_MENU_ITEM (check), state);
	if (func != NULL)
		g_signal_connect (GTK_OBJECT (check), "toggled", G_CALLBACK (func), data);
	gtk_widget_show (check);

	return check;
}

GtkWidget *add_menu_radio_button (GtkWidget *menu, GSList **group, gchar *label,
	gboolean state, void *func, gpointer data)
{
	GtkWidget *check = gtk_radio_menu_item_new_with_mnemonic (*group, label);
	*group = gtk_radio_menu_item_get_group (GTK_RADIO_MENU_ITEM (check));
	if (menu != NULL)
		gtk_menu_shell_append (GTK_MENU_SHELL (menu), check);
	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (check), state);
	if (func != NULL)
		g_signal_connect (GTK_OBJECT (check), "activate", G_CALLBACK (func), data);
	gtk_widget_show (check);

	return check;
}

GtkWidget *e2_menu_add_separator (GtkWidget *menu)
{
	GtkWidget *menu_item = gtk_separator_menu_item_new ();
	gtk_menu_append (GTK_MENU (menu), menu_item);
	gtk_widget_show (menu_item);
	return menu_item;
}

GtkWidget *add_menu_tear_off (GtkWidget *menu)
{
	GtkWidget *menu_item = gtk_tearoff_menu_item_new ();
	gtk_menu_append (GTK_MENU (menu), menu_item);
	gtk_widget_show (menu_item);
	return menu_item;
}

GtkWidget *add_submenu (GtkWidget *menu_bar, gchar *label_text, GtkWidget *menu)
{
	GtkWidget *menu_item;
	GtkWidget *label;

	menu_item = gtk_menu_item_new ();
	label = gtk_widget_new (GTK_TYPE_LABEL,
		                     "GtkWidget::visible", TRUE,
		                     "GtkWidget::parent", menu_item,
		                     "GtkMisc::xalign", 0.0,
		                     NULL);
	gtk_label_parse_uline (GTK_LABEL (label), label_text);
	gtk_menu_item_set_submenu (GTK_MENU_ITEM (menu_item), menu);
	gtk_menu_bar_append (GTK_MENU_BAR (menu_bar), menu_item);
	gtk_widget_show (menu_item);

	return menu_item;
}

GtkWidget *add_submenu2 (GtkWidget *menu, gchar *label_text)
{
	GtkWidget *menu_item = gtk_menu_item_new_with_mnemonic (label_text);
	gtk_menu_append (GTK_MENU (menu), menu_item);
	gtk_widget_show (menu_item);
	
	GtkWidget *submenu = gtk_menu_new ();
	gtk_menu_item_set_submenu (GTK_MENU_ITEM (menu_item), submenu);
	gtk_widget_show (submenu);

	return submenu;
}

void e2_menu_popup (GtkWidget *menu, gint button, guint32 time)
{
	g_signal_connect (G_OBJECT (menu), "selection-done",
		G_CALLBACK (e2_menu_destroy_cb), NULL);
	gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL, button, time);
}

void _e2_menu_popup (GtkWidget *menu, gint x, gint y, gint button, guint32 time)
{
	g_signal_connect (G_OBJECT (menu), "selection-done",
		G_CALLBACK (e2_menu_destroy_cb), NULL);
	gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL, button, time);
}

