/* $Id: e2_tree.c,v 1.9 2004/06/08 23:47:34 cairo Exp $
 * 
 * Copyright (C) 2004 tooar <tooar@gmx.net>
 * 
 * This file is part of emelFM2.
 * emelFM2 is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * emelFM2 is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with emelFM2; see the file COPYING. If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdlib.h>
#include <stdio.h>
#include <gtk/gtk.h>
#include "e2_tree.h"
#include "debug.h"

gboolean e2_tree_find_iter_from_str (GtkTreeModel *model, gint column,
	const gchar *search, GtkTreeIter *iter, gboolean with_children)
{
	if (model == NULL)
		return FALSE;
	gint num_columns = gtk_tree_model_get_n_columns (model);
	if (column > (num_columns - 1))
	{
		printd (WARN, "bad call of e2_tree_find_iter_from_str");
		printd (WARN, "requesting column %d of %d columns", column + 1, num_columns);
		return FALSE;
	}
	if (gtk_tree_model_get_column_type (model, column) != G_TYPE_STRING)
	{
		printd (WARN, "bad call of e2_tree_find_iter_from_str");
		printd (WARN, "column %d is not of type string", column + 1);
		return FALSE;
	}

	if (gtk_tree_model_get_iter_first (model, iter))
		return _e2_tree_find_iter_from_str (model, column, search, iter, with_children);
	return FALSE;
}

gboolean _e2_tree_find_iter_from_str (GtkTreeModel *model, gint column,
	const gchar *search, GtkTreeIter *iter, gboolean with_children)
{
	do
	{
		gchar *value;
		gtk_tree_model_get (model, iter, column, &value, -1);
		if (g_str_equal (value, search))
		{
			g_free (value);
			if (with_children)
			{
				GtkTreeIter iter2;
				if (gtk_tree_model_iter_children (model, &iter2, iter))
					return TRUE;
			} else
				return TRUE;
		}
		GtkTreeIter iter2;
		if (gtk_tree_model_iter_children (model, &iter2, iter))
		{
			if (_e2_tree_find_iter_from_str (model, column, search, &iter2, with_children))
			{
				*iter = iter2;
				return TRUE;
			}
		}
		g_free (value);
	} while (gtk_tree_model_iter_next (model, iter));
	return FALSE;
}

//FIXME: really silly way, do we really have to iterate through the whole
//tree model to get the last row?
gchar *e2_tree_get_last (GtkTreeModel *model, gint column, GtkTreeIter *iter)
{
	GtkTreePath *path = gtk_tree_path_new_first ();
	while (gtk_tree_model_get_iter (model, iter, path))
		gtk_tree_path_next (path);
	gtk_tree_path_prev (path);
	
	gchar *value = NULL;
	if (gtk_tree_model_get_iter (model, iter, path))
		gtk_tree_model_get (model, iter, column, &value, -1);
	gtk_tree_path_free (path);
	return value;
}

gint e2_tree_count (GtkTreeModel *model)
{
	gint count = 0;
	GtkTreeIter iter;
	if (!gtk_tree_model_get_iter_first (model, &iter))
		return count;
	do
	{
		count++;
	} while (gtk_tree_model_iter_next (model, &iter));
	return count;
}

GtkTreeRowReference *e2_tree_iter_to_ref (gpointer model, GtkTreeIter *iter)
{
	GtkTreePath *path = gtk_tree_model_get_path (GTK_TREE_MODEL (model),
		iter);
	GtkTreeRowReference *ref = gtk_tree_row_reference_new 
		(GTK_TREE_MODEL (model), path);
	gtk_tree_path_free (path);
	return ref;
}

gboolean e2_tree_ref_to_iter (gpointer model, GtkTreeRowReference *ref,
	GtkTreeIter *iter)
{
	if (gtk_tree_row_reference_valid (ref))
	{
		GtkTreePath *path = gtk_tree_row_reference_get_path (ref);
		gboolean retval = gtk_tree_model_get_iter (GTK_TREE_MODEL (model),
			iter, path);
		gtk_tree_path_free (path);
		return retval;
	}
	return FALSE;
}
